package javawrapper;

/**
 * This class organizes the seed predation behaviors.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>May 22, 2007: Added neighborhood seed predation (LEM)
 */

public class SeedPredationBehaviors extends BehaviorTypeBase {
	/**Neighborhood seed predation - masting "a"*/
	protected ModelVector mp_fNeighPredMastingA = new ModelVector(
			"Neighborhood Predation - Masting \"A\"",	"pr_neighPredMastingA", 
			"pr_npmaVal", 0, ModelVector.FLOAT);

	/**Neighborhood seed predation - masting "b"*/
	protected ModelVector mp_fNeighPredMastingB = new ModelVector(
			"Neighborhood Predation - Masting \"B\"",	"pr_neighPredMastingB", 
			"pr_npmbVal", 0, ModelVector.FLOAT);

	/**Neighborhood seed predation - masting "c"*/
	protected ModelVector mp_fNeighPredMastingC = new ModelVector(
			"Neighborhood Predation - Masting \"C\"",	"pr_neighPredMastingC", 
			"pr_npmcVal", 0, ModelVector.FLOAT);

	/**Neighborhood seed predation - non-masting "a"*/
	protected ModelVector mp_fNeighPredNonMastingA = new ModelVector(
			"Neighborhood Predation - Non-Masting \"A\"",	
			"pr_neighPredNonMastingA", "pr_npnmaVal", 0, ModelVector.FLOAT);
	
	/**Neighborhood seed predation - non-masting "b"*/
	protected ModelVector mp_fNeighPredNonMastingB = new ModelVector(
			"Neighborhood Predation - Non-Masting \"B\"",	
			"pr_neighPredNonMastingB", "pr_npnmbVal", 0, ModelVector.FLOAT);
	
	/**Neighborhood seed predation - non-masting "c"*/
	protected ModelVector mp_fNeighPredNonMastingC = new ModelVector(
			"Neighborhood Predation - Non-Masting \"C\"",	
			"pr_neighPredNonMastingC", "pr_npnmcVal", 0, ModelVector.FLOAT);

	/**Neighborhood seed predation - masting PCA loading factor - required for 
	 * all species*/
	protected ModelVector mp_fNeighPredMastingLoad = new ModelVector(
			"Neighborhood Predation - Masting Load Factor",	
			"pr_neighPredMastingPCALoadFactor", "pr_npmplfVal", 0, 
			ModelVector.FLOAT, true);

	/**Neighborhood seed predation - non-masting PCA loading factor - required
	 * for all species*/
	protected ModelVector mp_fNeighPredNonMastingLoad = new ModelVector(
			"Neighborhood Predation - Non-Masting Load Factor",	
			"pr_neighPredNonMastingPCALoadFactor", "pr_npnmplfVal", 0, 
			ModelVector.FLOAT, true);
	
	/**Neighborhood seed predation - density of seeds for masting, seeds/m2/yr*/
	protected ModelFloat m_fNeighPredMastingDensity = new ModelFloat(0,
      "Neighborhood Predation - Masting Seed Density, #/m2/yr", 
      "pr_neighPredMastingDensity");
	
	/**Neighborhood seed predation - min neighbor DBH*/
	protected ModelFloat m_fNeighPredMinDbh = new ModelFloat(0,
      "Neighborhood Predation - Minimum Neighbor DBH (cm)", 
      "pr_neighPredMinNeighDBH");
	
	/**Neighborhood seed predation - neighborhood search radius*/
	protected ModelFloat m_fNeighPredRadius = new ModelFloat(0,
      "Neighborhood Predation - Neighbor Search Radius (m)", 
      "pr_neighPredRadius");

  /**Functional response predation - predator initial density in number per 
   * square meter*/
  protected ModelFloat m_fFuncRespPredInitDensity = new ModelFloat(0,
      "Predator Initial Density - num/sq m", "pr_predatorInitialDensity");
  
  /**Functional response predation - max instantaneous rate at which predator 
   * abundance declines in the absense of food, in number of predators per 
   * week*/
  protected ModelFloat m_fFuncRespMaxInstDeclineRate = new ModelFloat(0,
      "Max Decline Rate in Absence of Food, predators/week",
      "pr_maxInstantaneousDeclineRate");
  
  /**Functional response predation - predator population's demographic 
   * efficiency*/
  protected ModelFloat m_fFuncRespDemogEfficiency = new ModelFloat(0,
      "Predator Population Demographic Efficiency", "pr_demographicEfficiency");
  
  /**Functional response predation - density-dependent coefficient*/
  protected ModelFloat m_fFunRespDensDepCoeff = new ModelFloat(0,
      "Density-Dependent Coefficient", "pr_densityDependentCoefficient");
  
  /**Functional response predation - maximum intake rate - number of seeds per
   * predator per day*/
  protected ModelFloat m_fFuncRespMaxIntake = new ModelFloat(0,
      "Max Intake Rate - seeds per predator per day", "pr_maxIntakeRate");
  
  /**Functional response predation - foraging efficiency*/
  protected ModelFloat m_fFuncRespForagingEff = new ModelFloat(0,
      "Foraging Efficiency", "pr_foragingEfficiency");
  
  /**Functional response predation - proportion of seeds, between 0 and 1, 
   * germinating each week in the germination period*/
  protected ModelFloat m_fFuncRespPropGerm = new ModelFloat(0,
      "Proportion of Seeds Removed Each Week During Germination Period",
      "pr_proportionGerminating");

  /**Functional response predation - number of weeks for which seedfall occurs*/
  protected ModelInt m_iFuncRespNumWeeksSeedFall = new ModelInt(0,
      "Number of Weeks in Which Seedfall Occurs", "pr_numWeeksSeedFall");
  
  /**Functional response predation - number of weeks to run model - must be 
   * less than or equal to 52*/
  protected ModelInt m_iFuncRespNumWeeksToModel = new ModelInt(0,
      "Weeks to Run Seed Predation Model (1 - 52)",
      "pr_numWeeksToModel");
  
  /**Functional response predation - week number in which germination begins*/
  protected ModelInt m_iFuncRespWeekGermStarts = new ModelInt(0,
      "Week During Predation Model Run Germination Begins",
      "pr_weekGerminationStarts");

  /**Functional response predation - output filename*/
  protected ModelString m_sFuncRespOutputFilename = new ModelString("",
      "Seed Predation Output Filename, If Desired", "pr_outputFilename");

  /**Functional response predation - whether or not to preserve predator 
   * densities between model timesteps (true) or re-initialize with predator
   * initial density (false)*/
  protected ModelEnum m_iPreservePredatorDensities = new ModelEnum(
		  new int[] {0, 1}, new String[] {"false", "true"},
      "Preserve Predator Densities Between SORTIE Timesteps",
      "pr_preservePredatorDensities");

  /**
   * Constructor.  Sets up the child behavior list and adds data appropriately.
   * @param oManager GUIManager object.
   * @throws ModelException passed through from called methods.  Should never
   * be thrown.
   */
  public SeedPredationBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Seed Predation", "seedPredation");

    m_sHelpID = "data.seed_predation_behaviors";
    mp_oChildBehaviors = new Behavior[2];

    //Set up the functional response predation behavior
    mp_oChildBehaviors[0] = new Behavior("funcResponsePredation",
                                         "Functional response seed predation",
                                         "functional response seed predation",
                                         (float)1);
    //Disallow for all but seeds
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEED, true);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SAPLING, false);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.ADULT, false);

    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespPredInitDensity);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespMaxInstDeclineRate);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespDemogEfficiency);
    mp_oChildBehaviors[0].AddRequiredData(m_fFunRespDensDepCoeff);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespMaxIntake);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespForagingEff);
    mp_oChildBehaviors[0].AddRequiredData(m_iFuncRespNumWeeksSeedFall);
    mp_oChildBehaviors[0].AddRequiredData(m_iFuncRespNumWeeksToModel);
    mp_oChildBehaviors[0].AddRequiredData(m_iFuncRespWeekGermStarts);
    mp_oChildBehaviors[0].AddRequiredData(m_iPreservePredatorDensities);
    mp_oChildBehaviors[0].AddRequiredData(m_fFuncRespPropGerm);
    mp_oChildBehaviors[0].AddRequiredData(m_sFuncRespOutputFilename);
    
    //Set up the neighborhood predation behavior
    mp_oChildBehaviors[1] = new Behavior("Neighborhood Predation",
                                         "Neighborhood Seed Predation",
                                         "Neighborhood Seed Predation",
                                         (float)1);
    //Disallow for all but seeds
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.SEED, true);
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.SAPLING, false);
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.ADULT, false);

    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredMastingA);
    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredMastingB);
    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredMastingC);
    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredNonMastingA);
    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredNonMastingB);
    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredNonMastingC);
    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredMastingLoad);
    mp_oChildBehaviors[1].AddRequiredData(mp_fNeighPredNonMastingLoad);
    mp_oChildBehaviors[1].AddRequiredData(m_fNeighPredMastingDensity);
    mp_oChildBehaviors[1].AddRequiredData(m_fNeighPredMinDbh);
    mp_oChildBehaviors[1].AddRequiredData(m_fNeighPredRadius);
    
    //Now set up the all data list
    mp_oAllData.add(m_fFuncRespPredInitDensity);
    mp_oAllData.add(m_fFuncRespMaxInstDeclineRate);
    mp_oAllData.add(m_fFuncRespDemogEfficiency);
    mp_oAllData.add(m_fFunRespDensDepCoeff);
    mp_oAllData.add(m_fFuncRespMaxIntake);
    mp_oAllData.add(m_fFuncRespForagingEff);
    mp_oAllData.add(m_iFuncRespNumWeeksSeedFall);
    mp_oAllData.add(m_iFuncRespNumWeeksToModel);
    mp_oAllData.add(m_iFuncRespWeekGermStarts);
    mp_oAllData.add(m_iPreservePredatorDensities);
    mp_oAllData.add(m_fFuncRespPropGerm);
    mp_oAllData.add(m_sFuncRespOutputFilename);
    mp_oAllData.add(mp_fNeighPredMastingA);
    mp_oAllData.add(mp_fNeighPredMastingB);
    mp_oAllData.add(mp_fNeighPredMastingC);
    mp_oAllData.add(mp_fNeighPredNonMastingA);
    mp_oAllData.add(mp_fNeighPredNonMastingB);
    mp_oAllData.add(mp_fNeighPredNonMastingC);
    mp_oAllData.add(mp_fNeighPredMastingLoad);
    mp_oAllData.add(mp_fNeighPredNonMastingLoad);
    mp_oAllData.add(m_fNeighPredMastingDensity);
    mp_oAllData.add(m_fNeighPredMinDbh);
    mp_oAllData.add(m_fNeighPredRadius);

    //Make the functional response seed predation grid
    Grid oNewGrid;

    //Create the data member
    DataMember[] p_oDataMembers = new DataMember[1];
    p_oDataMembers[0] = new DataMember("Number of predators", "num preds",
                                       DataMember.FLOAT);
    oNewGrid = new Grid("Seed Predators", p_oDataMembers, null, 8, 8);
    //Add to the functional response behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("funcResponsePredation").AddGrid(oNewGrid);
  }

  /**
   * Does nothing.
   * @param oPop Not needed.
   * @throws javawrapper.ModelException Not thrown.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void DoSetup(TreePopulation oPop) throws javawrapper.ModelException {}

  /**
   * Validates the dataset if seed predation is enabled.
   * @param oPop TreePopulation object.
   * @throws javawrapper.ModelException if:
   * <ul>
   * <li>Number of weeks of seedfall is less than or equal to zero</li>
   * <li>Initial predator density is less than or equal to zero</li>
   * <li>Number of weeks to run the model is less than 0 or greater than 52</li>
   * <li>Week to start germination is less than 0 or greater than 52</li>
   * <li>Proportion germinating is not a proportion</li>
   * <li>Neighborhood predation minimum neighbor DBH is less than zero</li>
   * <li>Neighborhood predation neighbor radius is less than zero</li>
   * <li>Neighborhood predation masting seed density threshold is less than
   * zero</li>
   * <li>Either seed predation behavior is present without a disperse behavior
   * also enabled</li>
   * </ul>
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>May 22, 2007: Added neighborhood seed predation (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws javawrapper.
      ModelException {

    if (!AnyBehaviorsEnabled()) {
      return;
    }

    if (GetBehaviorByKey("funcResponsePredation").m_bIsEnabled) {
    	MakeSureGreaterThan(m_iFuncRespNumWeeksSeedFall, 0);
    	MakeSureGreaterThan(m_fFuncRespPredInitDensity, 0);
    	MakeSureGreaterThan(m_iFuncRespNumWeeksToModel, 0);
    	MakeSureGreaterThanEqualTo(m_iFuncRespWeekGermStarts, 0);
      MakeSureIsProportion(m_fFuncRespPropGerm);
      MakeSureLessThan(m_iFuncRespNumWeeksToModel, 53);
      MakeSureLessThan(m_iFuncRespWeekGermStarts, 53);
      
      if (!m_oManager.GetDisperseBehaviors().AnyBehaviorsEnabled()) {
      	throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
      			                     "You must use a Disperse behavior with " +
      			                     GetBehaviorByKey("funcResponsePredation").GetDescriptor()));
      }
    }
   
    if (GetBehaviorByKey("Neighborhood Predation").m_bIsEnabled) {
    	MakeSureGreaterThan(m_fNeighPredMastingDensity, 0);
    	MakeSureGreaterThan(m_fNeighPredMinDbh, 0);
    	MakeSureGreaterThan(m_fNeighPredRadius, 0);
      if (!m_oManager.GetDisperseBehaviors().AnyBehaviorsEnabled()) {
      	throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
      			                     "You must use a Disperse behavior with " +
      			                     GetBehaviorByKey("Neighborhood Predation").GetDescriptor()));
      }
    }
  }  
}
