package javawrapper;
import java.io.File;

/**
 * Tests the DisperseBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

 */
public class TestDisperseBehaviors
    extends ModelTestCase {


  /**
   * Tests to make sure parameters are correctly displayed for each behavior.
   */
  public void testFormatDataForDisplay() {
    try {
      GUIManager oManager = new GUIManager(null);
      DisperseBehaviors oDisperse = null;
      TreePopulation oPop = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no disperse behaviors are enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      assertEquals(null, oDisperse.FormatDataForDisplay(oPop));

      //Case:  only non-spatial disperse is enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oDisperse.GetBehaviorByKey("non spatial disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Slope Mean Non-Spatial Seed Rain, seeds/m2/ha of BA/yr",
          "Intercept of Mean Non-Spatial Seed Rain, seeds/m2/yr",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution"
      };
      TestTable(oDisperse.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  no gap disperse is enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Weibull Canopy Beta",
          "STR/n for Stumps",
          "Beta for Stumps",
          "Lognormal Canopy Annual STR/n",
          "Lognormal Canopy Beta",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Weibull Canopy Annual STR/n",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution"
      };
      TestTable(oDisperse.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only masting disperse is enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution",
          "Masting Disperse - Masting CDF \"a\"",
          "Masting Disperse - Masting CDF \"b\"",
          "Masting Disperse - STR Draw PDF",
          "Masting Disperse - Non-Masting STR/n Mean",
          "Masting Disperse - Non-Masting STR/n Standard Deviation",
          "Masting Disperse - Masting STR/n Mean",
          "Masting Disperse - Masting STR/n Standard Deviation",
          "Masting Disperse - Non-Masting Beta",
          "Masting Disperse - Masting Beta",
          "Masting Disperse - Masting Weibull Dispersal",
          "Masting Disperse - Masting Weibull Theta",
          "Masting Disperse - Masting Lognormal X0",
          "Masting Disperse - Masting Lognormal Xb",
          "Masting Disperse - Masting Group",
          "Masting Disperse - Stochastic STR Draw Frequency",
          "Masting Disperse - Mast Proportion Participating (0-1)",
          "Masting Disperse - Non-Mast Proportion Participating (0-1)"
      };
      TestTable(oDisperse.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case:  only gap disperse is enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oDisperse.GetBehaviorByKey("gap disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Weibull Gap Annual STR/n",
          "Weibull Gap Beta",
          "Weibull Gap Theta",
          "Weibull Gap Dispersal",
          "Maximum Parent Trees Allowed in Gap Cell",
          "Lognormal Gap Annual STR/n",
          "Lognormal Gap Beta",
          "Lognormal Gap Xb",
          "Lognormal Gap X0",
          "Lognormal Canopy Annual STR/n",
          "Lognormal Canopy Beta",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Gap Function Used",
          "STR/n for Stumps",
          "Beta for Stumps",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Weibull Canopy Beta",
          "Weibull Canopy Annual STR/n",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution"
      };
      TestTable(oDisperse.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  all are enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      int i;
      for (i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
        oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oDisperse.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
            SpeciesTypeCombo(
            0, 3, oPop));
      }
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Maximum Parent Trees Allowed in Gap Cell",
          "Weibull Gap Annual STR/n",
          "Weibull Gap Beta",
          "Weibull Gap Theta",
          "Weibull Gap Dispersal",
          "Lognormal Gap Annual STR/n",
          "Lognormal Gap Beta",
          "Lognormal Gap Xb",
          "Lognormal Gap X0",
          "Lognormal Canopy Annual STR/n",
          "Lognormal Canopy Beta",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Gap Function Used",
          "STR/n for Stumps",
          "Beta for Stumps",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Weibull Canopy Beta",
          "Weibull Canopy Annual STR/n",
          "Slope Mean Non-Spatial Seed Rain, seeds/m2/ha of BA/yr",
          "Intercept of Mean Non-Spatial Seed Rain, seeds/m2/yr",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution",
          "Azimuth Direction of Max Dispersal Distance, in rad",
          "Amplitude of Anisotropic Effect",
          "Maximum Search Distance for Neighbor Parents, in m",
          "Masting Disperse - Masting CDF \"a\"",
          "Masting Disperse - Masting CDF \"b\"",
          "Masting Disperse - STR Draw PDF",
          "Masting Disperse - Non-Masting STR/n Mean",
          "Masting Disperse - Non-Masting STR/n Standard Deviation",
          "Masting Disperse - Masting STR/n Mean",
          "Masting Disperse - Masting STR/n Standard Deviation",
          "Masting Disperse - Non-Masting Beta",
          "Masting Disperse - Masting Beta",
          "Masting Disperse - Masting Weibull Dispersal",
          "Masting Disperse - Masting Weibull Theta",
          "Masting Disperse - Masting Lognormal X0",
          "Masting Disperse - Masting Lognormal Xb",
          "Masting Disperse - Masting Group",
          "Masting Disperse - Stochastic STR Draw Frequency",
          "Masting Disperse - Mast Proportion Participating (0-1)",
          "Masting Disperse - Non-Mast Proportion Participating (0-1)"
      };
      TestTable(oDisperse.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for disperse.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests species changes. Even though DisperseBehaviors doesn't explicitly
   * have code for changing species, this makes sure that it is treated
   * correctly.
   */
  public void testChangeOfSpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      ModelEnum oEnum;
      oManager.ClearCurrentData();
      sFileName = WriteAllDisperseXMLFile();
      oManager.InputXMLParameterFile(sFileName);

      int WEIB = DisperseBehaviors.WEIBULL;
      int LOGN = DisperseBehaviors.LOGNORMAL;
      int GAP = DisperseBehaviors.GAP;
      int CAN = DisperseBehaviors.CANOPY;


      //Verify initial file read
      DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();

      assertEquals(((Float)oDisperse.mp_fMinDbhForReproduction.GetValue().elementAt(0)).floatValue(), 15.0, 0.001);
      assertEquals(((Float)oDisperse.mp_fMinDbhForReproduction.GetValue().elementAt(1)).floatValue(), 16.0, 0.001);
      assertEquals(((Float)oDisperse.mp_fMinDbhForReproduction.GetValue().elementAt(2)).floatValue(), 17.0, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][CAN].GetValue().elementAt(0)).floatValue(), 19.325, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][CAN].GetValue().elementAt(1)).floatValue(), 29.325, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 12.257, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][GAP].GetValue().elementAt(0)).floatValue(), 14.894, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][GAP].GetValue().elementAt(1)).floatValue(), 12.4, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 6.628, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][CAN].GetValue().elementAt(0)).floatValue(), 6.628, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][CAN].GetValue().elementAt(1)).floatValue(), 19.325, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 17.206, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][GAP].GetValue().elementAt(0)).floatValue(), 7.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][GAP].GetValue().elementAt(1)).floatValue(), 21.5, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 69.5, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][CAN].GetValue().elementAt(0)).floatValue(), 2.0, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][CAN].GetValue().elementAt(1)).floatValue(), 2.1, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 2.2, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][GAP].GetValue().elementAt(0)).floatValue(), 2.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][GAP].GetValue().elementAt(1)).floatValue(), 2.4, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 2.5, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][CAN].GetValue().elementAt(0)).floatValue(), 2.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][CAN].GetValue().elementAt(1)).floatValue(), 2.8, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 2.9, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][GAP].GetValue().elementAt(0)).floatValue(), 5.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][GAP].GetValue().elementAt(1)).floatValue(), 5.8, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 5.9, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][CAN].GetValue().elementAt(0)).floatValue(), 3.0, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][CAN].GetValue().elementAt(1)).floatValue(), 3.1, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 3.2, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][GAP].GetValue().elementAt(0)).floatValue(), 3.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][GAP].GetValue().elementAt(1)).floatValue(), 3.4, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 3.5, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][CAN].GetValue().elementAt(0)).floatValue(), 3.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][CAN].GetValue().elementAt(1)).floatValue(), 3.8, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 3.9, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][GAP].GetValue().elementAt(0)).floatValue(), 5.1, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][GAP].GetValue().elementAt(1)).floatValue(), 5.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 5.3, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][CAN].GetValue().elementAt(0)).floatValue(), 1.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][CAN].GetValue().elementAt(1)).floatValue(), -5.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 9.61, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][GAP].GetValue().elementAt(0)).floatValue(), 1.32, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][GAP].GetValue().elementAt(1)).floatValue(), 5.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 6.93, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][CAN].GetValue().elementAt(0)).floatValue(), 2.82, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][CAN].GetValue().elementAt(1)).floatValue(), 3.32, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 4.93, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][GAP].GetValue().elementAt(0)).floatValue(), 1.71, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][GAP].GetValue().elementAt(1)).floatValue(), 1.81, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 1.91, 0.001);

      assertEquals(((Float)oDisperse.mp_fStumpSTR.GetValue().elementAt(0)).floatValue(), 12.257, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpSTR.GetValue().elementAt(1)).floatValue(), 22.257, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpSTR.GetValue().elementAt(2)).floatValue(), 32.257, 0.001);

      assertEquals(((Float)oDisperse.mp_fStumpBeta.GetValue().elementAt(0)).floatValue(), 4.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpBeta.GetValue().elementAt(1)).floatValue(), 4.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpBeta.GetValue().elementAt(2)).floatValue(), 4.4, 0.001);

      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals(((Float)oDisperse.mp_fSlopeOfLambda.GetValue().elementAt(0)).floatValue(), 0, 0.001);
      assertEquals(((Float)oDisperse.mp_fSlopeOfLambda.GetValue().elementAt(1)).floatValue(), 5, 0.001);
      assertEquals(((Float)oDisperse.mp_fSlopeOfLambda.GetValue().elementAt(2)).floatValue(), 10, 0.001);

      assertEquals(((Float)oDisperse.mp_fInterceptOfLambda.GetValue().elementAt(0)).floatValue(), 0.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fInterceptOfLambda.GetValue().elementAt(1)).floatValue(), 50, 0.001);
      assertEquals(((Float)oDisperse.mp_fInterceptOfLambda.GetValue().elementAt(2)).floatValue(), 8, 0.001);

      assertEquals(((Float)oDisperse.mp_fDirectionMaxDispersal.GetValue().elementAt(0)).floatValue(), 0.352, 0.001);
      assertEquals(((Float)oDisperse.mp_fDirectionMaxDispersal.GetValue().elementAt(1)).floatValue(), 2.70, 0.001);
      assertEquals(((Float)oDisperse.mp_fDirectionMaxDispersal.GetValue().elementAt(2)).floatValue(), 1, 0.001);

      assertEquals(((Float)oDisperse.mp_fAnisotropicAmplitude.GetValue().elementAt(0)).floatValue(), 1.352, 0.001);
      assertEquals(((Float)oDisperse.mp_fAnisotropicAmplitude.GetValue().elementAt(1)).floatValue(), 8.70, 0.001);
      assertEquals(((Float)oDisperse.mp_fAnisotropicAmplitude.GetValue().elementAt(2)).floatValue(), 2, 0.001);

      assertEquals(((Float)oDisperse.mp_fStandardDeviation.GetValue().elementAt(0)).floatValue(), -4.1, 0.001);
      assertEquals(((Float)oDisperse.mp_fStandardDeviation.GetValue().elementAt(1)).floatValue(), -4.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fStandardDeviation.GetValue().elementAt(2)).floatValue(), -4.3, 0.001);

      assertEquals(((Float)oDisperse.mp_fClumpingParameter.GetValue().elementAt(0)).floatValue(), -0.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fClumpingParameter.GetValue().elementAt(1)).floatValue(), -0.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fClumpingParameter.GetValue().elementAt(2)).floatValue(), -0.6, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastingA.GetValue().elementAt(0)).floatValue(), 0.00001, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastingA.GetValue().elementAt(1)).floatValue(), 100, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastingA.GetValue().elementAt(2)).floatValue(), 1000, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastingB.GetValue().elementAt(0)).floatValue(), -5, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastingB.GetValue().elementAt(1)).floatValue(), -6, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastingB.GetValue().elementAt(2)).floatValue(), -7, 0.001);
      
      oEnum = (ModelEnum) oDisperse.mp_iMastSTRDrawPDF.GetValue().elementAt(0);
      assertEquals(0, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastSTRDrawPDF.GetValue().elementAt(1);
      assertEquals(2, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastSTRDrawPDF.GetValue().elementAt(2);
      assertEquals(3, oEnum.GetValue());
      
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRMean .GetValue().elementAt(0)).floatValue(), 550, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRMean.GetValue().elementAt(1)).floatValue(), 35, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRMean.GetValue().elementAt(2)).floatValue(), 10001, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRStdDev .GetValue().elementAt(0)).floatValue(), 10, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRStdDev.GetValue().elementAt(1)).floatValue(), 20, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRStdDev.GetValue().elementAt(2)).floatValue(), 30, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastSTRMean .GetValue().elementAt(0)).floatValue(), 385, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastSTRMean.GetValue().elementAt(1)).floatValue(), 214, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastSTRMean.GetValue().elementAt(2)).floatValue(), 126, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastSTRStdDev .GetValue().elementAt(0)).floatValue(), 40, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastSTRStdDev.GetValue().elementAt(1)).floatValue(), 50, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastSTRStdDev.GetValue().elementAt(2)).floatValue(), 60, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastNonMastBeta .GetValue().elementAt(0)).floatValue(), 1, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastBeta.GetValue().elementAt(1)).floatValue(), 3.5, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastBeta.GetValue().elementAt(2)).floatValue(), 2, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastBeta .GetValue().elementAt(0)).floatValue(), 2, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastBeta.GetValue().elementAt(1)).floatValue(), 2.5, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastBeta.GetValue().elementAt(2)).floatValue(), 1, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastWeibDisp.GetValue().elementAt(0)).floatValue(), 1.76E-04, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastWeibDisp.GetValue().elementAt(1)).floatValue(), 1.82E-04, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastWeibDisp.GetValue().elementAt(2)).floatValue(), 9.61E-05, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastWeibTheta.GetValue().elementAt(0)).floatValue(), 3, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastWeibTheta.GetValue().elementAt(1)).floatValue(), 3.1, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastWeibTheta.GetValue().elementAt(2)).floatValue(), 3.2, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalX0.GetValue().elementAt(0)).floatValue(), 10.92, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalX0.GetValue().elementAt(1)).floatValue(), 11.92, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalX0.GetValue().elementAt(2)).floatValue(), 12.92, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalXb.GetValue().elementAt(0)).floatValue(), 0.181, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalXb.GetValue().elementAt(1)).floatValue(), 0.281, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalXb.GetValue().elementAt(2)).floatValue(), 0.381, 0.001);
      
      oEnum = (ModelEnum) oDisperse.mp_iMastGroupID.GetValue().elementAt(0);
      assertEquals(1, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastGroupID.GetValue().elementAt(1);
      assertEquals(2, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastGroupID.GetValue().elementAt(2);
      assertEquals(1, oEnum.GetValue());
      
      oEnum = (ModelEnum) oDisperse.mp_iMastDrawPerSpecies.GetValue().elementAt(0);
      assertEquals(0, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastDrawPerSpecies.GetValue().elementAt(1);
      assertEquals(1, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastDrawPerSpecies.GetValue().elementAt(2);
      assertEquals(0, oEnum.GetValue());
      
      assertEquals(((Float)oDisperse.mp_fMastMastPropParticipating.GetValue().elementAt(0)).floatValue(), 0.24, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastPropParticipating.GetValue().elementAt(1)).floatValue(), 0.66, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastPropParticipating.GetValue().elementAt(2)).floatValue(), 0.75, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastNonMastPropParticipating.GetValue().elementAt(0)).floatValue(), 0.01, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastPropParticipating.GetValue().elementAt(1)).floatValue(), 0.13, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastPropParticipating.GetValue().elementAt(2)).floatValue(), 0.43, 0.001);

      //Now change the species
      String[] sNewSpecies = new String[] {
          "Species 5",
          "Species 4",
          "Species 3",
          "Species 1"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      oDisperse = oManager.GetDisperseBehaviors();
      assertEquals(((Float)oDisperse.mp_fMinDbhForReproduction.GetValue().elementAt(3)).floatValue(), 15.0, 0.001);
      assertEquals(((Float)oDisperse.mp_fMinDbhForReproduction.GetValue().elementAt(2)).floatValue(), 17.0, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][CAN].GetValue().elementAt(3)).floatValue(), 19.325, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 12.257, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][GAP].GetValue().elementAt(3)).floatValue(), 14.894, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 6.628, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][CAN].GetValue().elementAt(3)).floatValue(), 6.628, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 17.206, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][GAP].GetValue().elementAt(3)).floatValue(), 7.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 69.5, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][CAN].GetValue().elementAt(3)).floatValue(), 2.0, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 2.2, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][GAP].GetValue().elementAt(3)).floatValue(), 2.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 2.5, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][CAN].GetValue().elementAt(3)).floatValue(), 2.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 2.9, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][GAP].GetValue().elementAt(3)).floatValue(), 5.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 5.9, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][CAN].GetValue().elementAt(3)).floatValue(), 3.0, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 3.2, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][GAP].GetValue().elementAt(3)).floatValue(), 3.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 3.5, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][CAN].GetValue().elementAt(3)).floatValue(), 3.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 3.9, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][GAP].GetValue().elementAt(3)).floatValue(), 5.1, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 5.3, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][CAN].GetValue().elementAt(3)).floatValue(), 1.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 9.61, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][GAP].GetValue().elementAt(3)).floatValue(), 1.32, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 6.93, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][CAN].GetValue().elementAt(3)).floatValue(), 2.82, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 4.93, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][GAP].GetValue().elementAt(3)).floatValue(), 1.71, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 1.91, 0.001);

      assertEquals(((Float)oDisperse.mp_fStumpSTR.GetValue().elementAt(3)).floatValue(), 12.257, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpSTR.GetValue().elementAt(2)).floatValue(), 32.257, 0.001);

      assertEquals(((Float)oDisperse.mp_fStumpBeta.GetValue().elementAt(3)).floatValue(), 4.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpBeta.GetValue().elementAt(2)).floatValue(), 4.4, 0.001);

      int i;
      assertEquals(4, oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().size());
      for (i = 0; i < 4; i++) {
        assertTrue(null != oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().elementAt(3);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 0);

      assertEquals(4, oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().size());
      for (i = 0; i < 4; i++) {
        assertTrue(null != oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().elementAt(3);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals(((Float)oDisperse.mp_fSlopeOfLambda.GetValue().elementAt(3)).floatValue(), 0, 0.001);
      assertEquals(((Float)oDisperse.mp_fSlopeOfLambda.GetValue().elementAt(2)).floatValue(), 10, 0.001);

      assertEquals(((Float)oDisperse.mp_fInterceptOfLambda.GetValue().elementAt(3)).floatValue(), 0.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fInterceptOfLambda.GetValue().elementAt(2)).floatValue(), 8, 0.001);

      assertEquals(((Float)oDisperse.mp_fDirectionMaxDispersal.GetValue().elementAt(3)).floatValue(), 0.352, 0.001);
      assertEquals(((Float)oDisperse.mp_fDirectionMaxDispersal.GetValue().elementAt(2)).floatValue(), 1, 0.001);

      assertEquals(((Float)oDisperse.mp_fAnisotropicAmplitude.GetValue().elementAt(3)).floatValue(), 1.352, 0.001);
      assertEquals(((Float)oDisperse.mp_fAnisotropicAmplitude.GetValue().elementAt(2)).floatValue(), 2, 0.001);

      assertEquals(((Float)oDisperse.mp_fStandardDeviation.GetValue().elementAt(3)).floatValue(), -4.1, 0.001);
      assertEquals(((Float)oDisperse.mp_fStandardDeviation.GetValue().elementAt(2)).floatValue(), -4.3, 0.001);

      assertEquals(((Float)oDisperse.mp_fClumpingParameter.GetValue().elementAt(3)).floatValue(), -0.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fClumpingParameter.GetValue().elementAt(2)).floatValue(), -0.6, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastingA.GetValue().elementAt(3)).floatValue(), 0.00001, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastingA.GetValue().elementAt(2)).floatValue(), 1000, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastingB.GetValue().elementAt(3)).floatValue(), -5, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastingB.GetValue().elementAt(2)).floatValue(), -7, 0.001);
      
      for (i = 0; i < 4; i++) {
        assertTrue(null != oDisperse.mp_iMastSTRDrawPDF.GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oDisperse.mp_iMastSTRDrawPDF.GetValue().elementAt(3);
      assertEquals(0, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastSTRDrawPDF.GetValue().elementAt(2);
      assertEquals(3, oEnum.GetValue());
      
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRMean .GetValue().elementAt(3)).floatValue(), 550, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRMean.GetValue().elementAt(2)).floatValue(), 10001, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRStdDev .GetValue().elementAt(3)).floatValue(), 10, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastSTRStdDev.GetValue().elementAt(2)).floatValue(), 30, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastSTRMean .GetValue().elementAt(3)).floatValue(), 385, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastSTRMean.GetValue().elementAt(2)).floatValue(), 126, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastSTRStdDev .GetValue().elementAt(3)).floatValue(), 40, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastSTRStdDev.GetValue().elementAt(2)).floatValue(), 60, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastNonMastBeta .GetValue().elementAt(3)).floatValue(), 1, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastBeta.GetValue().elementAt(2)).floatValue(), 2, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastBeta .GetValue().elementAt(3)).floatValue(), 2, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastBeta.GetValue().elementAt(2)).floatValue(), 1, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastWeibDisp.GetValue().elementAt(3)).floatValue(), 1.76E-04, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastWeibDisp.GetValue().elementAt(2)).floatValue(), 9.61E-05, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastWeibTheta.GetValue().elementAt(3)).floatValue(), 3, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastWeibTheta.GetValue().elementAt(2)).floatValue(), 3.2, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalX0.GetValue().elementAt(3)).floatValue(), 10.92, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalX0.GetValue().elementAt(2)).floatValue(), 12.92, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalXb.GetValue().elementAt(3)).floatValue(), 0.181, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastLognormalXb.GetValue().elementAt(2)).floatValue(), 0.381, 0.001);
      
      for (i = 0; i < 4; i++) {
        oEnum = (ModelEnum) oDisperse.mp_iMastGroupID.GetValue().elementAt(i);
        assertTrue(null != oEnum);
        assertEquals(4, oEnum.mp_iAllowedValues.length);
      }
      oEnum = (ModelEnum) oDisperse.mp_iMastGroupID.GetValue().elementAt(0);
      assertEquals(1, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastGroupID.GetValue().elementAt(1);
      assertEquals(2, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastGroupID.GetValue().elementAt(3);
      assertEquals(1, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastGroupID.GetValue().elementAt(2);
      assertEquals(1, oEnum.GetValue());
      
      for (i = 0; i < 4; i++) {
        assertTrue(null != oDisperse.mp_iMastDrawPerSpecies.GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oDisperse.mp_iMastDrawPerSpecies.GetValue().elementAt(3);
      assertEquals(0, oEnum.GetValue());
      oEnum = (ModelEnum) oDisperse.mp_iMastDrawPerSpecies.GetValue().elementAt(2);
      assertEquals(0, oEnum.GetValue());
      
      assertEquals(((Float)oDisperse.mp_fMastMastPropParticipating.GetValue().elementAt(3)).floatValue(), 0.24, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastMastPropParticipating.GetValue().elementAt(2)).floatValue(), 0.75, 0.001);
      
      assertEquals(((Float)oDisperse.mp_fMastNonMastPropParticipating.GetValue().elementAt(3)).floatValue(), 0.01, 0.001);
      assertEquals(((Float)oDisperse.mp_fMastNonMastPropParticipating.GetValue().elementAt(2)).floatValue(), 0.43, 0.001);


      System.out.println("Change of species test succeeded.");
    }
    catch (ModelException oErr) {
      fail("XML volume reading test failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests species copying. Even though DisperseBehaviors doesn't explicitly
   * have code for copying species, this makes sure that it is treated
   * correctly.
   */
  public void testCopySpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      ModelEnum oEnum;
      oManager.ClearCurrentData();
      sFileName = WriteAllDisperseXMLFile();
      oManager.InputXMLParameterFile(sFileName);

      int WEIB = DisperseBehaviors.WEIBULL;
      int LOGN = DisperseBehaviors.LOGNORMAL;
      int GAP = DisperseBehaviors.GAP;
      int CAN = DisperseBehaviors.CANOPY;

      //Now copy a species
      oManager.GetTreePopulation().DoCopySpecies("Species_3", "Species_1");
      DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();

      assertEquals(((Float)oDisperse.mp_fMinDbhForReproduction.GetValue().elementAt(0)).floatValue(), 17.0, 0.001);
      assertEquals(((Float)oDisperse.mp_fMinDbhForReproduction.GetValue().elementAt(1)).floatValue(), 16.0, 0.001);
      assertEquals(((Float)oDisperse.mp_fMinDbhForReproduction.GetValue().elementAt(2)).floatValue(), 17.0, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][CAN].GetValue().elementAt(0)).floatValue(), 12.257, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][CAN].GetValue().elementAt(1)).floatValue(), 29.325, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 12.257, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][GAP].GetValue().elementAt(0)).floatValue(), 6.628, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][GAP].GetValue().elementAt(1)).floatValue(), 12.4, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 6.628, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][CAN].GetValue().elementAt(0)).floatValue(), 17.206, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][CAN].GetValue().elementAt(1)).floatValue(), 19.325, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 17.206, 0.001);

      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][GAP].GetValue().elementAt(0)).floatValue(), 69.5, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][GAP].GetValue().elementAt(1)).floatValue(), 21.5, 0.001);
      assertEquals(((Float)oDisperse.mp_fSTR[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 69.5, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][CAN].GetValue().elementAt(0)).floatValue(), 2.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][CAN].GetValue().elementAt(1)).floatValue(), 2.1, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 2.2, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][GAP].GetValue().elementAt(0)).floatValue(), 2.5, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][GAP].GetValue().elementAt(1)).floatValue(), 2.4, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 2.5, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][CAN].GetValue().elementAt(0)).floatValue(), 2.9, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][CAN].GetValue().elementAt(1)).floatValue(), 2.8, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 2.9, 0.001);

      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][GAP].GetValue().elementAt(0)).floatValue(), 5.9, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][GAP].GetValue().elementAt(1)).floatValue(), 5.8, 0.001);
      assertEquals(((Float)oDisperse.mp_fBeta[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 5.9, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][CAN].GetValue().elementAt(0)).floatValue(), 3.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][CAN].GetValue().elementAt(1)).floatValue(), 3.1, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 3.2, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][GAP].GetValue().elementAt(0)).floatValue(), 3.5, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][GAP].GetValue().elementAt(1)).floatValue(), 3.4, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 3.5, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][CAN].GetValue().elementAt(0)).floatValue(), 3.9, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][CAN].GetValue().elementAt(1)).floatValue(), 3.8, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 3.9, 0.001);

      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][GAP].GetValue().elementAt(0)).floatValue(), 5.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][GAP].GetValue().elementAt(1)).floatValue(), 5.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fThetaOrXb[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 5.3, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][CAN].GetValue().elementAt(0)).floatValue(), 9.61, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][CAN].GetValue().elementAt(1)).floatValue(), -5.7, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][CAN].GetValue().elementAt(2)).floatValue(), 9.61, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][GAP].GetValue().elementAt(0)).floatValue(), 6.93, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][GAP].GetValue().elementAt(1)).floatValue(), 5.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[WEIB][GAP].GetValue().elementAt(2)).floatValue(), 6.93, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][CAN].GetValue().elementAt(0)).floatValue(), 4.93, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][CAN].GetValue().elementAt(1)).floatValue(), 3.32, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][CAN].GetValue().elementAt(2)).floatValue(), 4.93, 0.001);

      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][GAP].GetValue().elementAt(0)).floatValue(), 1.91, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][GAP].GetValue().elementAt(1)).floatValue(), 1.81, 0.001);
      assertEquals(((Float)oDisperse.mp_fDispOrX0[LOGN][GAP].GetValue().elementAt(2)).floatValue(), 1.91, 0.001);

      assertEquals(((Float)oDisperse.mp_fStumpSTR.GetValue().elementAt(0)).floatValue(), 32.257, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpSTR.GetValue().elementAt(1)).floatValue(), 22.257, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpSTR.GetValue().elementAt(2)).floatValue(), 32.257, 0.001);

      assertEquals(((Float)oDisperse.mp_fStumpBeta.GetValue().elementAt(0)).floatValue(), 4.4, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpBeta.GetValue().elementAt(1)).floatValue(), 4.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fStumpBeta.GetValue().elementAt(2)).floatValue(), 4.4, 0.001);

      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[GAP].GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oDisperse.mp_iWhichFunctionUsed[CAN].GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals(((Float)oDisperse.mp_fSlopeOfLambda.GetValue().elementAt(0)).floatValue(), 10, 0.001);
      assertEquals(((Float)oDisperse.mp_fSlopeOfLambda.GetValue().elementAt(1)).floatValue(), 5, 0.001);
      assertEquals(((Float)oDisperse.mp_fSlopeOfLambda.GetValue().elementAt(2)).floatValue(), 10, 0.001);

      assertEquals(((Float)oDisperse.mp_fInterceptOfLambda.GetValue().elementAt(0)).floatValue(), 8, 0.001);
      assertEquals(((Float)oDisperse.mp_fInterceptOfLambda.GetValue().elementAt(1)).floatValue(), 50, 0.001);
      assertEquals(((Float)oDisperse.mp_fInterceptOfLambda.GetValue().elementAt(2)).floatValue(), 8, 0.001);

      assertEquals(((Float)oDisperse.mp_fDirectionMaxDispersal.GetValue().elementAt(0)).floatValue(), 1, 0.001);
      assertEquals(((Float)oDisperse.mp_fDirectionMaxDispersal.GetValue().elementAt(1)).floatValue(), 2.70, 0.001);
      assertEquals(((Float)oDisperse.mp_fDirectionMaxDispersal.GetValue().elementAt(2)).floatValue(), 1, 0.001);

      assertEquals(((Float)oDisperse.mp_fAnisotropicAmplitude.GetValue().elementAt(0)).floatValue(), 2, 0.001);
      assertEquals(((Float)oDisperse.mp_fAnisotropicAmplitude.GetValue().elementAt(1)).floatValue(), 8.70, 0.001);
      assertEquals(((Float)oDisperse.mp_fAnisotropicAmplitude.GetValue().elementAt(2)).floatValue(), 2, 0.001);

      assertEquals(((Float)oDisperse.mp_fStandardDeviation.GetValue().elementAt(0)).floatValue(), -4.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fStandardDeviation.GetValue().elementAt(1)).floatValue(), -4.2, 0.001);
      assertEquals(((Float)oDisperse.mp_fStandardDeviation.GetValue().elementAt(2)).floatValue(), -4.3, 0.001);

      assertEquals(((Float)oDisperse.mp_fClumpingParameter.GetValue().elementAt(0)).floatValue(), -0.6, 0.001);
      assertEquals(((Float)oDisperse.mp_fClumpingParameter.GetValue().elementAt(1)).floatValue(), -0.3, 0.001);
      assertEquals(((Float)oDisperse.mp_fClumpingParameter.GetValue().elementAt(2)).floatValue(), -0.6, 0.001);

      System.out.println("Copy species test succeeded.");
    }
    catch (ModelException oErr) {
      fail("XML volume reading test failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }
  
  /**
   * Tests the ValidateData function.
   */
  public void testValidateData() {
    try {
      GUIManager oManager = new GUIManager(null);
      String sFileName = null;
      TreePopulation oPop = null;
      Behavior oBeh = null;
      try {
        //Valid file
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetDisperseBehaviors().ValidateData(oManager.GetTreePopulation());
        new File(sFileName).delete();
      }
      catch (ModelException oErr) {
        fail("Disperse validation failed with message " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
    
      //Case: canopy thetas are greater than 50 and lognormal X0s and Xbs 
      //are = 0, but there are no applicable behaviors enabled; no error 
      //should be thrown
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        //Disable gap, nongap, and masting spatial disperse
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.ClearSpeciesTypeCombos();
        oBeh.m_bIsEnabled = false;
        oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
        oBeh.ClearSpeciesTypeCombos();
        oBeh.m_bIsEnabled = false;
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.ClearSpeciesTypeCombos();
        oBeh.m_bIsEnabled = false;
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(70));
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(0));
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
      }
      catch (ModelException oErr) {
        fail("Disperse validation failed with message " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: canopy, gap, and masting Xbs and X0s are = 0, but nobody uses 
      //lognormal function
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().size(); i++) {
          ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(i);
          oEnum.SetValue(DisperseBehaviors.WEIBULL);
          oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(i);
          oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(i, oEnum);
        }
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(new Float(0));
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().add(new Float(0));
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(new Float(0));
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().add(new Float(0));
        oDisperse.mp_fMastMastLognormalX0.GetValue().remove(0);
        oDisperse.mp_fMastMastLognormalX0.GetValue().add(new Float(0));
        oDisperse.mp_fMastMastLognormalXb.GetValue().remove(0);
        oDisperse.mp_fMastMastLognormalXb.GetValue().add(new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
      }
      catch (ModelException oErr) {
        fail("Disperse validation failed with message " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: Lognormal canopy Xbs = 0, and only gap disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad Xb canopy values " +
            "when only gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Canopy Xb") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: Lognormal canopy X0s = 0, and only gap disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad X0 canopy values " +
            "when only gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Canopy X0") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: Lognormal canopy Xbs = 0, and only non-gap disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad Xb canopy values " +
            "when only non-gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Canopy Xb") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: Lognormal canopy X0s = 0, and only non-gap disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad X0 canopy values " +
            "when only non-gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Canopy X0") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: Lognormal canopy Xbs = 0, and only masting disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad Xb canopy values " +
            "when only masting disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Canopy Xb") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: Lognormal canopy X0s = 0, and only masting disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad X0 canopy values " +
            "when only masting disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Canopy X0") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: gap lognormal Xbs = 0, and gap disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.WEIBULL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.GAP].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.GAP].GetValue().add(0, oEnum);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad Xb gap values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Gap Xb") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: gap lognormal X0s = 0, and gap disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.WEIBULL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.GAP].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.GAP].GetValue().add(0, oEnum);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fDispOrX0[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad X0 gap values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Gap X0") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
     //Case: masting lognormal Xbs = 0, and masting disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oDisperse.mp_fMastMastLognormalXb.GetValue().remove(0);
        oDisperse.mp_fMastMastLognormalXb.GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad Xb masting values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Masting Lognormal Xb") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: masting lognormal X0s = 0, and masting disperse is enabled
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        for (int i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
          oDisperse.mp_oChildBehaviors[i].ClearSpeciesTypeCombos();
          oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = false;
        }
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        ModelEnum oEnum = (ModelEnum)oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().elementAt(0);
        oEnum.SetValue(DisperseBehaviors.LOGNORMAL);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_iWhichFunctionUsed[DisperseBehaviors.CANOPY].GetValue().add(0, oEnum);
        oDisperse.mp_fMastMastLognormalX0.GetValue().remove(0);
        oDisperse.mp_fMastMastLognormalX0.GetValue().add(0, new Float(0));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad X0 masting values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Masting Lognormal X0") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
    
      //Case: canopy thetas are greater than 50, and only gap disperse is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad theta canopy values " +
            "when only gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Weibull Canopy Theta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: canopy thetas are greater than 50, and only non gap disperse 
      //is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad theta canopy values " +
            "when only non gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Weibull Canopy Theta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: canopy thetas are greater than 50, and only masting disperse 
      //is enabled
      try {
        oManager.ClearCurrentData();
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad theta canopy values " +
            "when only masting disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Weibull Canopy Theta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: gap thetas are greater than 50, but there are no applicable
      //behaviors enabled; no error should be thrown
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        //Disable gap disperse
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.ClearSpeciesTypeCombos();
        oBeh.m_bIsEnabled = false;
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.GAP].GetValue().add(0, new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
      }
      catch (ModelException oErr) {
        fail("Disperse validation failed with message " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
    
      //Case: gap thetas are greater than 50, and gap disperse is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fThetaOrXb[DisperseBehaviors.WEIBULL][DisperseBehaviors.GAP].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad theta gap values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Weibull Gap Theta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: masting thetas are greater than 50, and masting disperse 
      //is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fMastMastWeibTheta.GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad theta masting values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Masting Weibull Theta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: weibull and lognormal canopy betas are greater than 25, but there
      //are no applicable behaviors enabled; no error should be thrown
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        //Disable gap, nongap, and masting spatial disperse
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.ClearSpeciesTypeCombos();
        oBeh.m_bIsEnabled = false;
        oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
        oBeh.ClearSpeciesTypeCombos();
        oBeh.m_bIsEnabled = false;
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.ClearSpeciesTypeCombos();
        oBeh.m_bIsEnabled = false;
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(70));
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(0, new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
      }
      catch (ModelException oErr) {
        fail("Disperse validation failed with message " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
    
      //Case: weibull canopy betas are greater than 25, and only gap disperse 
      //is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad beta weibull canopy " +
            "values when only gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Weibull Canopy Beta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: lognormal canopy betas are greater than 25, and only gap disperse 
      //is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad beta lognormal canopy " +
            "values when only gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Canopy Beta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: weibull canopy betas are greater than 25, and only non-gap 
      //disperse is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad beta weibull canopy " +
            "values when only non gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Weibull Canopy Beta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: lognormal canopy betas are greater than 25, and only non gap 
      //disperse is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad beta lognormal canopy " +
            "values when only non gap disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Canopy Beta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: weibull canopy betas are greater than 25, and only masting spatial 
      //disperse is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.CANOPY].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad beta weibull canopy " +
            "values when only masting disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Weibull Canopy Beta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: lognormal canopy betas are greater than 25, and only masting 
      //disperse is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.CANOPY].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad beta lognormal canopy " +
            "values when only masting disperse was enabled.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Canopy Beta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: gap betas are greater than 25, but there are no applicable
      //behaviors enabled; no error should be thrown
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        //Disable gap disperse
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.ClearSpeciesTypeCombos();
        oBeh.m_bIsEnabled = false;
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.GAP].GetValue().add(0, new Float(70));
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().add(0, new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
      }
      catch (ModelException oErr) {
        fail("Disperse validation failed with message " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
    
      //Case: weibull gap betas are greater than 25, and gap disperse is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.WEIBULL][DisperseBehaviors.GAP].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad weibull beta gap values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Weibull Gap Beta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: masting betas are greater than 25, and masting disperse 
      //is enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("masting spatial disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fMastMastBeta.GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad masting beta values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Masting Beta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: lognormal gap betas are greater than 25, and gap disperse is 
      //enabled
      try {
        sFileName = WriteNoDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oBeh = oDisperse.GetBehaviorByKey("gap disperse");
        oBeh.m_bIsEnabled = true;
        oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().remove(0);
        oDisperse.mp_fBeta[DisperseBehaviors.LOGNORMAL][DisperseBehaviors.GAP].GetValue().add(new Float(70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad lognormal beta gap values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Lognormal Gap Beta") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: a value in mp_fMinDbhForReproduction is negative
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oDisperse.mp_fMinDbhForReproduction.GetValue().remove(0);
        oDisperse.mp_fMinDbhForReproduction.GetValue().add(0, new Float(-70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch negative reproduction DBH" +
            " values.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Minimum DBH for Reproduction") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: max search radius for anisotropic disperse is not a positive 
      //number
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oDisperse.m_fMaxSearchRadius.SetValue(-10);
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch negative anisotropic search" +
            " radius.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Maximum Search Distance") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: max search radius for anisotropic disperse is not less than 
      //half the shortest plot distance
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oDisperse.m_fMaxSearchRadius.SetValue(1000);
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch too-large anisotropic " +
            "search radius.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Maximum Search Distance") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: the direction of max dispersal is not between 0 and 2PI
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oDisperse.mp_fDirectionMaxDispersal.GetValue().remove(0);
        oDisperse.mp_fDirectionMaxDispersal.GetValue().add(0, new Float(-70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad anisotropic direction.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Azimuth Direction") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: masting fraction participating is not a proportion
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oDisperse.mp_fMastMastPropParticipating.GetValue().remove(0);
        oDisperse.mp_fMastMastPropParticipating.GetValue().add(0, new Float(-70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad mast proportion " +
            "participating value.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Mast Proportion Participating") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
      //Case: non-masting fraction participating is not a proportion
      try {
        sFileName = WriteAllDisperseXMLFile();
        oManager.InputXMLParameterFile(sFileName);
        oPop = oManager.GetTreePopulation();
        DisperseBehaviors oDisperse = oManager.GetDisperseBehaviors();
        oDisperse.mp_fMastNonMastPropParticipating.GetValue().remove(0);
        oDisperse.mp_fMastNonMastPropParticipating.GetValue().add(0, new Float(-70));
        oManager.GetDisperseBehaviors().ValidateData(oPop);
        fail("Disperse validation failed to catch bad non mast proportion " +
            "participating value.");
      }
      catch (ModelException oErr) {
        if (oErr.getMessage().indexOf("Non-Mast Proportion Participating") == -1)
          fail("Incorrect error. Message: " + oErr.getMessage());
      }
      finally {
        new File(sFileName).delete();
      }
      
    }  
    catch (ModelException oErr) {
      fail("Disperse validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Writes a file with no disperse settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>August 18, 2006: Created (LEM)
   */
  protected String WriteNoDisperseXMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>50</timesteps>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with all disperse settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>August 18, 2006: Created (LEM)
   */
  protected String WriteAllDisperseXMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>50</timesteps>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>gap disperse</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>non-gap disperse</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Stump\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>non spatial disperse</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>masting spatial disperse</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>anisotropic disperse</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<disperse>");
    oOut.write("<di_minDbhForReproduction>");
    oOut.write("<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>");
    oOut.write("<di_mdfrVal species=\"Species_2\">16.0</di_mdfrVal>");
    oOut.write("<di_mdfrVal species=\"Species_3\">17.0</di_mdfrVal>");
    oOut.write("</di_minDbhForReproduction>");
    oOut.write("<di_weibullCanopySTR>");
    oOut.write("<di_wcsVal species=\"Species_1\">19.325</di_wcsVal>");
    oOut.write("<di_wcsVal species=\"Species_2\">29.325</di_wcsVal>");
    oOut.write("<di_wcsVal species=\"Species_3\">12.257</di_wcsVal>");
    oOut.write("</di_weibullCanopySTR>");
    oOut.write("<di_weibullCanopyBeta>");
    oOut.write("<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>");
    oOut.write("<di_wcbVal species=\"Species_2\">2.1</di_wcbVal>");
    oOut.write("<di_wcbVal species=\"Species_3\">2.2</di_wcbVal>");
    oOut.write("</di_weibullCanopyBeta>");
    oOut.write("<di_weibullCanopyDispersal>");
    oOut.write("<di_wcdVal species=\"Species_1\">1.7</di_wcdVal>");
    oOut.write("<di_wcdVal species=\"Species_2\">-5.7</di_wcdVal>");
    oOut.write("<di_wcdVal species=\"Species_3\">9.61</di_wcdVal>");
    oOut.write("</di_weibullCanopyDispersal>");
    oOut.write("<di_weibullCanopyTheta>");
    oOut.write("<di_wctVal species=\"Species_1\">3.0</di_wctVal>");
    oOut.write("<di_wctVal species=\"Species_2\">3.1</di_wctVal>");
    oOut.write("<di_wctVal species=\"Species_3\">3.2</di_wctVal>");
    oOut.write("</di_weibullCanopyTheta>");
    oOut.write("<di_weibullGapSTR>");
    oOut.write("<di_wgsVal species=\"Species_1\">14.894</di_wgsVal>");
    oOut.write("<di_wgsVal species=\"Species_2\">12.4</di_wgsVal>");
    oOut.write("<di_wgsVal species=\"Species_3\">6.628</di_wgsVal>");
    oOut.write("</di_weibullGapSTR>");
    oOut.write("<di_weibullGapBeta>");
    oOut.write("<di_wgbVal species=\"Species_1\">2.3</di_wgbVal>");
    oOut.write("<di_wgbVal species=\"Species_2\">2.4</di_wgbVal>");
    oOut.write("<di_wgbVal species=\"Species_3\">2.5</di_wgbVal>");
    oOut.write("</di_weibullGapBeta>");
    oOut.write("<di_weibullGapDispersal>");
    oOut.write("<di_wgdVal species=\"Species_1\">1.32</di_wgdVal>");
    oOut.write("<di_wgdVal species=\"Species_2\">5.2</di_wgdVal>");
    oOut.write("<di_wgdVal species=\"Species_3\">6.93</di_wgdVal>");
    oOut.write("</di_weibullGapDispersal>");
    oOut.write("<di_weibullGapTheta>");
    oOut.write("<di_wgtVal species=\"Species_1\">3.3</di_wgtVal>");
    oOut.write("<di_wgtVal species=\"Species_2\">3.4</di_wgtVal>");
    oOut.write("<di_wgtVal species=\"Species_3\">3.5</di_wgtVal>");
    oOut.write("</di_weibullGapTheta>");
    oOut.write("<di_lognormalCanopySTR>");
    oOut.write("<di_lcsVal species=\"Species_1\">6.628</di_lcsVal>");
    oOut.write("<di_lcsVal species=\"Species_2\">19.325</di_lcsVal>");
    oOut.write("<di_lcsVal species=\"Species_3\">17.206</di_lcsVal>");
    oOut.write("</di_lognormalCanopySTR>");
    oOut.write("<di_lognormalCanopyBeta>");
    oOut.write("<di_lcbVal species=\"Species_1\">2.7</di_lcbVal>");
    oOut.write("<di_lcbVal species=\"Species_2\">2.8</di_lcbVal>");
    oOut.write("<di_lcbVal species=\"Species_3\">2.9</di_lcbVal>");
    oOut.write("</di_lognormalCanopyBeta>");
    oOut.write("<di_lognormalCanopyX0>");
    oOut.write("<di_lcx0Val species=\"Species_1\">2.82</di_lcx0Val>");
    oOut.write("<di_lcx0Val species=\"Species_2\">3.32</di_lcx0Val>");
    oOut.write("<di_lcx0Val species=\"Species_3\">4.93</di_lcx0Val>");
    oOut.write("</di_lognormalCanopyX0>");
    oOut.write("<di_lognormalCanopyXb>");
    oOut.write("<di_lcxbVal species=\"Species_1\">3.7</di_lcxbVal>");
    oOut.write("<di_lcxbVal species=\"Species_2\">3.8</di_lcxbVal>");
    oOut.write("<di_lcxbVal species=\"Species_3\">3.9</di_lcxbVal>");
    oOut.write("</di_lognormalCanopyXb>");
    oOut.write("<di_lognormalGapSTR>");
    oOut.write("<di_lgsVal species=\"Species_1\">7.2</di_lgsVal>");
    oOut.write("<di_lgsVal species=\"Species_2\">21.5</di_lgsVal>");
    oOut.write("<di_lgsVal species=\"Species_3\">69.5</di_lgsVal>");
    oOut.write("</di_lognormalGapSTR>");
    oOut.write("<di_lognormalGapBeta>");
    oOut.write("<di_lgbVal species=\"Species_1\">5.7</di_lgbVal>");
    oOut.write("<di_lgbVal species=\"Species_2\">5.8</di_lgbVal>");
    oOut.write("<di_lgbVal species=\"Species_3\">5.9</di_lgbVal>");
    oOut.write("</di_lognormalGapBeta>");
    oOut.write("<di_lognormalGapX0>");
    oOut.write("<di_lgx0Val species=\"Species_1\">1.71</di_lgx0Val>");
    oOut.write("<di_lgx0Val species=\"Species_2\">1.81</di_lgx0Val>");
    oOut.write("<di_lgx0Val species=\"Species_3\">1.91</di_lgx0Val>");
    oOut.write("</di_lognormalGapX0>");
    oOut.write("<di_lognormalGapXb>");
    oOut.write("<di_lgxbVal species=\"Species_1\">5.1</di_lgxbVal>");
    oOut.write("<di_lgxbVal species=\"Species_2\">5.2</di_lgxbVal>");
    oOut.write("<di_lgxbVal species=\"Species_3\">5.3</di_lgxbVal>");
    oOut.write("</di_lognormalGapXb>");
    oOut.write("<di_suckerSTR>");
    oOut.write("<di_ssVal species=\"Species_1\">12.257</di_ssVal>");
    oOut.write("<di_ssVal species=\"Species_2\">22.257</di_ssVal>");
    oOut.write("<di_ssVal species=\"Species_3\">32.257</di_ssVal>");
    oOut.write("</di_suckerSTR>");
    oOut.write("<di_suckerBeta>");
    oOut.write("<di_sbVal species=\"Species_1\">4.2</di_sbVal>");
    oOut.write("<di_sbVal species=\"Species_2\">4.3</di_sbVal>");
    oOut.write("<di_sbVal species=\"Species_3\">4.4</di_sbVal>");
    oOut.write("</di_suckerBeta>");
    oOut.write("<di_maxGapDensity>0</di_maxGapDensity>");
    oOut.write("<di_gapFunction>");
    oOut.write("<di_gfVal species=\"Species_1\">0</di_gfVal>");
    oOut.write("<di_gfVal species=\"Species_2\">1</di_gfVal>");
    oOut.write("<di_gfVal species=\"Species_3\">0</di_gfVal>");
    oOut.write("</di_gapFunction>");
    oOut.write("<di_canopyFunction>");
    oOut.write("<di_cfVal species=\"Species_1\">1</di_cfVal>");
    oOut.write("<di_cfVal species=\"Species_2\">0</di_cfVal>");
    oOut.write("<di_cfVal species=\"Species_3\">1</di_cfVal>");
    oOut.write("</di_canopyFunction>");
    oOut.write("<di_nonSpatialSlopeOfLambda>");
    oOut.write("<di_nssolVal species=\"Species_1\">0</di_nssolVal>");
    oOut.write("<di_nssolVal species=\"Species_2\">5</di_nssolVal>");
    oOut.write("<di_nssolVal species=\"Species_3\">10</di_nssolVal>");
    oOut.write("</di_nonSpatialSlopeOfLambda>");
    oOut.write("<di_nonSpatialInterceptOfLambda>");
    oOut.write("<di_nsiolVal species=\"Species_1\">0.3</di_nsiolVal>");
    oOut.write("<di_nsiolVal species=\"Species_2\">50</di_nsiolVal>");
    oOut.write("<di_nsiolVal species=\"Species_3\">8</di_nsiolVal>");
    oOut.write("</di_nonSpatialInterceptOfLambda>");
    oOut.write("<di_standardDeviation>");
    oOut.write("<di_sdVal species=\"Species_1\">-4.1</di_sdVal>");
    oOut.write("<di_sdVal species=\"Species_2\">-4.2</di_sdVal>");
    oOut.write("<di_sdVal species=\"Species_3\">-4.3</di_sdVal>");
    oOut.write("</di_standardDeviation>");
    oOut.write("<di_clumpingParameter>");
    oOut.write("<di_cpVal species=\"Species_1\">-0.2</di_cpVal>");
    oOut.write("<di_cpVal species=\"Species_2\">-0.3</di_cpVal>");
    oOut.write("<di_cpVal species=\"Species_3\">-0.6</di_cpVal>");
    oOut.write("</di_clumpingParameter>");
    oOut.write("<di_directionMaxDispersal>");
    oOut.write("<di_dmdVal species=\"Species_1\">0.352</di_dmdVal>");
    oOut.write("<di_dmdVal species=\"Species_2\">2.70</di_dmdVal>");
    oOut.write("<di_dmdVal species=\"Species_3\">1</di_dmdVal>");
    oOut.write("</di_directionMaxDispersal>");
    oOut.write("<di_anisotropicAmplitude>");
    oOut.write("<di_aaVal species=\"Species_1\">1.352</di_aaVal>");
    oOut.write("<di_aaVal species=\"Species_2\">8.70</di_aaVal>");
    oOut.write("<di_aaVal species=\"Species_3\">2</di_aaVal>");
    oOut.write("</di_anisotropicAmplitude>");
    oOut.write("<di_maxSearchRadius>20.0</di_maxSearchRadius>");
    oOut.write("<di_seedDistributionMethod>0</di_seedDistributionMethod>");
    oOut.write("<di_mastCDFA>");
    oOut.write("<di_mcdfaVal species=\"Species_1\">0.00001</di_mcdfaVal>");
    oOut.write("<di_mcdfaVal species=\"Species_2\">100</di_mcdfaVal>");
    oOut.write("<di_mcdfaVal species=\"Species_3\">1000</di_mcdfaVal>");
    oOut.write("</di_mastCDFA>");
    oOut.write("<di_mastCDFB>");
    oOut.write("<di_mcdfbVal species=\"Species_1\">-5</di_mcdfbVal>");
    oOut.write("<di_mcdfbVal species=\"Species_2\">-6</di_mcdfbVal>");
    oOut.write("<di_mcdfbVal species=\"Species_3\">-7</di_mcdfbVal>");
    oOut.write("</di_mastCDFB>");
    oOut.write("<di_mastSTRPDF>");
    oOut.write("<di_mstrpdfVal species=\"Species_1\">0</di_mstrpdfVal>");
    oOut.write("<di_mstrpdfVal species=\"Species_2\">2</di_mstrpdfVal>");
    oOut.write("<di_mstrpdfVal species=\"Species_3\">3</di_mstrpdfVal>");
    oOut.write("</di_mastSTRPDF>");
    oOut.write("<di_spatialSTR>");
    oOut.write("<di_sstrVal species=\"Species_1\">550</di_sstrVal>");
    oOut.write("<di_sstrVal species=\"Species_2\">35</di_sstrVal>");
    oOut.write("<di_sstrVal species=\"Species_3\">10001</di_sstrVal>");
    oOut.write("</di_spatialSTR>");
    oOut.write("<di_spatialSTRStdDev>");
    oOut.write("<di_sstrsdVal species=\"Species_1\">10</di_sstrsdVal>");
    oOut.write("<di_sstrsdVal species=\"Species_2\">20</di_sstrsdVal>");
    oOut.write("<di_sstrsdVal species=\"Species_3\">30</di_sstrsdVal>");
    oOut.write("</di_spatialSTRStdDev>");
    oOut.write("<di_mastingSTR>");
    oOut.write("<di_mstrVal species=\"Species_1\">385</di_mstrVal>");
    oOut.write("<di_mstrVal species=\"Species_2\">214</di_mstrVal>");
    oOut.write("<di_mstrVal species=\"Species_3\">126</di_mstrVal>");
    oOut.write("</di_mastingSTR>");
    oOut.write("<di_mastingSTRStdDev>");
    oOut.write("<di_mstrsdVal species=\"Species_1\">40</di_mstrsdVal>");
    oOut.write("<di_mstrsdVal species=\"Species_2\">50</di_mstrsdVal>");
    oOut.write("<di_mstrsdVal species=\"Species_3\">60</di_mstrsdVal>");
    oOut.write("</di_mastingSTRStdDev>");
    oOut.write("<di_spatialBeta>");
    oOut.write("<di_sbVal species=\"Species_1\">1</di_sbVal>");
    oOut.write("<di_sbVal species=\"Species_2\">3.5</di_sbVal>");
    oOut.write("<di_sbVal species=\"Species_3\">2</di_sbVal>");
    oOut.write("</di_spatialBeta>");
    oOut.write("<di_mastingBeta>");
    oOut.write("<di_mbVal species=\"Species_1\">2</di_mbVal>");
    oOut.write("<di_mbVal species=\"Species_2\">2.5</di_mbVal>");
    oOut.write("<di_mbVal species=\"Species_3\">1</di_mbVal>");
    oOut.write("</di_mastingBeta>");
    oOut.write("<di_weibullMastingDispersal>");
    oOut.write("<di_wmdVal species=\"Species_1\">1.76E-04</di_wmdVal>");
    oOut.write("<di_wmdVal species=\"Species_2\">1.82E-04</di_wmdVal>");
    oOut.write("<di_wmdVal species=\"Species_3\">9.61E-05</di_wmdVal>");
    oOut.write("</di_weibullMastingDispersal>");
    oOut.write("<di_weibullMastingTheta>");
    oOut.write("<di_wmtVal species=\"Species_1\">3</di_wmtVal>");
    oOut.write("<di_wmtVal species=\"Species_2\">3.1</di_wmtVal>");
    oOut.write("<di_wmtVal species=\"Species_3\">3.2</di_wmtVal>");
    oOut.write("</di_weibullMastingTheta>");
    oOut.write("<di_lognormalMastingX0>");
    oOut.write("<di_lmx0Val species=\"Species_1\">10.92</di_lmx0Val>");
    oOut.write("<di_lmx0Val species=\"Species_2\">11.92</di_lmx0Val>");
    oOut.write("<di_lmx0Val species=\"Species_3\">12.92</di_lmx0Val>");
    oOut.write("</di_lognormalMastingX0>");
    oOut.write("<di_lognormalMastingXb>");
    oOut.write("<di_lmxbVal species=\"Species_1\">0.181</di_lmxbVal>");
    oOut.write("<di_lmxbVal species=\"Species_2\">0.281</di_lmxbVal>");
    oOut.write("<di_lmxbVal species=\"Species_3\">0.381</di_lmxbVal>");
    oOut.write("</di_lognormalMastingXb>");
    oOut.write("<di_mastGroup>");
    oOut.write("<di_mgVal species=\"Species_1\">1</di_mgVal>");
    oOut.write("<di_mgVal species=\"Species_2\">2</di_mgVal>");
    oOut.write("<di_mgVal species=\"Species_3\">1</di_mgVal>");
    oOut.write("</di_mastGroup>");
    oOut.write("<di_mastDrawPerSpecies>");
    oOut.write("<di_mdpsVal species=\"Species_1\">0</di_mdpsVal>");
    oOut.write("<di_mdpsVal species=\"Species_2\">1</di_mdpsVal>");
    oOut.write("<di_mdpsVal species=\"Species_3\">0</di_mdpsVal>");
    oOut.write("</di_mastDrawPerSpecies>");
    oOut.write("<di_mastPropParticipating>");
    oOut.write("<di_mppVal species=\"Species_1\">0.24</di_mppVal>");
    oOut.write("<di_mppVal species=\"Species_2\">0.66</di_mppVal>");
    oOut.write("<di_mppVal species=\"Species_3\">0.75</di_mppVal>");
    oOut.write("</di_mastPropParticipating>");
    oOut.write("<di_spatialPropParticipating>");
    oOut.write("<di_sppVal species=\"Species_1\">0.01</di_sppVal>");
    oOut.write("<di_sppVal species=\"Species_2\">0.13</di_sppVal>");
    oOut.write("<di_sppVal species=\"Species_3\">0.43</di_sppVal>");
    oOut.write("</di_spatialPropParticipating>");
    oOut.write("</disperse>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}
