package javawrapper;

import java.io.FileWriter;
import java.io.File;

/**
 * Tests the GrowthBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestGrowthBehaviors
    extends ModelTestCase {

  /**
   * This test makes sure that lambdas are reset correctly if species are
   * changed.
   */
  public void testDoSetup() {
    try {
      ModelVector p_oData;
      int i;

      //First - set up the way it is
      GUIManager oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU", "ACSA"});
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      oGrowth.DoSetup(oManager.GetTreePopulation());

      //Set some lambda values
      Float[] p_fFloat = new Float[] {new Float(0.01), new Float(0.02)};
    Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
    for (i = 0; i < oBeh.GetNumberOfRequiredDataObjects(); i++) {
      ModelData oData = (ModelData) oBeh.GetRequiredData(i);
      String sDescriptor = oData.GetDescriptor().toLowerCase();
      if (sDescriptor.indexOf("lambda") > -1) {
        p_oData = (ModelVector) oBeh.GetRequiredData(i);
        oGrowth.SetVectorValues(p_oData, p_fFloat);
        break;
      }
    }

      p_fFloat[0] = new Float(0.11);
      p_fFloat[1] = new Float(0.12);
      for (i = 0; i < oBeh.GetNumberOfRequiredDataObjects(); i++) {
      ModelData oData = (ModelData) oBeh.GetRequiredData(i);
      String sDescriptor = oData.GetDescriptor().toLowerCase();
      if (sDescriptor.indexOf("lambda") > -1) {
        p_oData = (ModelVector) oBeh.GetRequiredData(i+1);
        oGrowth.SetVectorValues(p_oData, p_fFloat);
        break;
      }
    }

      //Make sure there are ACRU and ACSA lambda neighbors in the whole
      //data list
      boolean bFoundNCIACRU = false,
          bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("nci");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      //************************
       // Now remove a species
       //************************
      oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU"});
      //Make sure there are ACRU lambda neighbors only in the whole
      //data list
      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
            //Make sure it has the values we set for it
            p_oData = (ModelVector) oData;
            assertEquals(0.01,((Float)p_oData.GetValue().elementAt(0)).doubleValue(), 0.001);
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            fail("Found NCI ACSA in all data when it should be gone.");
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("nci");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            fail("Found NCI ACSA in behavior when it should be gone.");
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);

      //************************
       //Reset our species back to normal
       //************************
      oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU", "ACSA"});
      //Make sure there are ACRU and ACSA lambda neighbors in the whole
      //data list
      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("nci");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
  }

  /**
   * Tests that the correct parameters are displayed for each behavior.
   */
  public void testFormatDataForDisplay() {
    try {
      GUIManager oManager = new GUIManager(null);
      TreePopulation oPop;
      GrowthBehaviors oGrowth = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no growth behaviors are enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oPop = oManager.GetTreePopulation();
      oGrowth = oManager.GetGrowthBehaviors();
      assertEquals(null, oGrowth.FormatDataForDisplay(oPop));

      //Case:  only absolute radial growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absradial");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Radial Growth in mm/yr",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only absolute basal area growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Area Growth in sq. cm/yr",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only absolute unlimited growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oPop = oManager.GetTreePopulation();
      oGrowth = oManager.GetGrowthBehaviors();
      oBeh = oGrowth.GetBehaviorByKey("absunlim");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative radial growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relradial");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative basal area growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative unlimited growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relunlim");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Slope of Growth Response (S)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only constant radial growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("constradial");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only constant basal area growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("constba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only nci growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Alpha",
          "NCI Beta",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI Damage Effect - Medium Storm Damage (0-1)",
          "NCI Damage Effect - Complete Storm Damage (0-1)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "NCI Minimum Neighbor DBH, in cm",
          "Species 1 NCI Lambda Neighbors",
          "Species 2 NCI Lambda Neighbors"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only ba nci growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("nciba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Minimum Neighbor DBH, in cm"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only logistic growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("logistic growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
          "Logistic - Diam Shape Param 1 (b)",
          "Logistic - Diam Shape Param 2 (c)",
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only simple linear growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Simple Linear - Diam Intercept in mm/yr (a)",
          "Simple Linear - Diam Slope (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only linear bi-level growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Linear Bi-Level - Intercept for High-Light Growth (a)",
          "Linear Bi-Level - Slope for Low-Light Growth (b)",
          "Linear Bi-Level - Intercept for Low-Light Growth (a)",
          "Linear Bi-Level - Slope for High-Light Growth (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only size-dependent logistic growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("size dependent logistic growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Size Dep. Logistic - Diam Intercept (a)",
          "Size Dep. Logistic - Diam Slope (b)",
          "Size Dep. Logistic - Diam Shape Param 1 (c)",
          "Size Dep. Logistic - Diam Shape Param 2 (d)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only lognormal growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("lognormal growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
          "Lognormal - Diam Shape Parameter (b)",
          "Lognormal - Diam Effect of Shade (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only shaded linear growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("shaded linear");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Shaded Linear - Diam Intercept in mm/yr (a)",
          "Shaded Linear - Diam Slope (b)",
          "Shaded Linear - Diam Shade Exponent (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only double-resource relative growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("double resource");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Double resource - Influence of Resource (C)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Puerto Rico storm bi-level growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "PR Storm Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "PR Storm Bi-Level - High-Light \"a\"",
          "PR Storm Bi-Level - High-Light \"b\"",
          "PR Storm Bi-Level - Slope for Low-Light Growth (b)",
          "PR Storm Bi-Level - Intercept for Low-Light Growth (a)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);


      //Case:  only absolute radial growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absradial diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Radial Growth in mm/yr",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only absolute basal area growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Area Growth in sq. cm/yr",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only absolute unlimited growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absunlim diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative radial growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relradial diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative basal area growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative unlimited growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relunlim diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Slope of Growth Response (S)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only constant radial growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("constradial diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only constant basal area growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("constba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only nci growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("nci diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Alpha",
          "NCI Beta",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI Damage Effect - Medium Storm Damage (0-1)",
          "NCI Damage Effect - Complete Storm Damage (0-1)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "NCI Minimum Neighbor DBH, in cm",
          "Species 1 NCI Lambda Neighbors",
          "Species 2 NCI Lambda Neighbors"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only ba nci growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("nciba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Minimum Neighbor DBH, in cm"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only logistic growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("logistic growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
          "Logistic - Diam Shape Param 1 (b)",
          "Logistic - Diam Shape Param 2 (c)",
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only linear bi-level growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear bi-level diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Linear Bi-Level - Intercept for High-Light Growth (a)",
          "Linear Bi-Level - Slope for Low-Light Growth (b)",
          "Linear Bi-Level - Intercept for Low-Light Growth (a)",
          "Linear Bi-Level - Slope for High-Light Growth (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Simple Linear growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Simple Linear - Diam Intercept in mm/yr (a)",
          "Simple Linear - Diam Slope (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only size-dependent logistic growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey(
          "size dependent logistic growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Size Dep. Logistic - Diam Intercept (a)",
          "Size Dep. Logistic - Diam Slope (b)",
          "Size Dep. Logistic - Diam Shape Param 1 (c)",
          "Size Dep. Logistic - Diam Shape Param 2 (d)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only lognormal growth, Diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("lognormal growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
          "Lognormal - Diam Shape Parameter (b)",
          "Lognormal - Diam Effect of Shade (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Shaded Linear growth, Diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("shaded linear diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Shaded Linear - Diam Intercept in mm/yr (a)",
          "Shaded Linear - Diam Slope (b)",
          "Shaded Linear - Diam Shade Exponent (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only double-resource relative growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("double resource diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Double resource - Influence of Resource (C)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only height incrementer is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("height incrementer");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      assertEquals(null, oGrowth.FormatDataForDisplay(oPop));

      //Case:  only logistic growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("logistic growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Logistic - Asymptotic Height Growth - Full Light in cm/yr (a)",
          "Logistic - Height Shape Param 1 (b)",
          "Logistic - Height Shape Param 2 (c)",
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Simple Linear growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Simple Linear - Height Intercept in cm/yr (a)",
          "Simple Linear - Height Slope (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only size-dependent logistic growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey(
          "size dependent logistic growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Size Dep. Logistic - Height Intercept (a)",
          "Size Dep. Logistic - Height Slope (b)",
          "Size Dep. Logistic - Height Shape Param 1 (c)",
          "Size Dep. Logistic - Height Shape Param 2 (d)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only lognormal growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("lognormal growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Lognormal - Height Growth Increment at Diam 36, in cm/yr (a)",
          "Lognormal - Height Shape Parameter (b)",
          "Lognormal - Height Effect of Shade (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Shaded Linear growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("shaded linear height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Shaded Linear - Height Intercept in cm/yr (a)",
          "Shaded Linear - Height Slope (b)",
          "Shaded Linear - Height Shade Exponent (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Shaded Linear growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Lognormal Bi-Level - X0 for Low-Light Growth",
          "Lognormal Bi-Level - Xb for Low-Light Growth",
          "Lognormal Bi-Level - X0 for High-Light Growth",
          "Lognormal Bi-Level - Xb for High-Light Growth",
          "Lognormal Bi-Level - Max Growth in High Light (m)",
          "Lognormal Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Lognormal Bi-Level - Max Growth in Low Light (m)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only PR semi-stochastic diameter growth is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("pr semi-stochastic diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "PR - Height Threshold for Stochastic Growth (m)",
          "PR - \"a\" Parameter for Deterministic Growth",
          "PR - \"b\" Parameter for Deterministic Growth",
          "PR - Mean DBH (cm) for Stochastic Growth",
          "PR - DBH Standard Deviation for Stochastic Growth"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only browsed relative growth, diam with auto height, is enabled
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("browsed relative growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Browsed Asymptotic Diameter Growth (A)",
          "Browsed Slope of Growth Response (S)",
          "Browsed Diameter Exponent"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only browsed relative growth, diam only, is enabled
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("browsed relative growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Browsed Asymptotic Diameter Growth (A)",
          "Browsed Slope of Growth Response (S)",
          "Browsed Diameter Exponent"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);




      //Case:  all are enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      int i;
      for (i = 0; i < oGrowth.mp_oChildBehaviors.length; i++) {
        oGrowth.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oGrowth.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new SpeciesTypeCombo(
            0, 3, oPop));
      }
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Alpha",
          "NCI Beta",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI Damage Effect - Medium Storm Damage (0-1)",
          "NCI Damage Effect - Complete Storm Damage (0-1)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "NCI Minimum Neighbor DBH, in cm",
          "Species 1 NCI Lambda Neighbors",
          "Species 2 NCI Lambda Neighbors",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Slope of Growth Response (S)",
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Radial Growth in mm/yr",
          "Adult Constant Area Growth in sq. cm/yr",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor",
          "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
          "Logistic - Asymptotic Height Growth - Full Light in cm/yr (a)",
          "Logistic - Diam Shape Param 1 (b)",
          "Logistic - Height Shape Param 1 (b)",
          "Logistic - Diam Shape Param 2 (c)",
          "Logistic - Height Shape Param 2 (c)",
          "Simple Linear - Diam Intercept in mm/yr (a)",
          "Simple Linear - Height Intercept in cm/yr (a)",
          "Simple Linear - Diam Slope (b)",
          "Simple Linear - Height Slope (b)",
          "Shaded Linear - Diam Intercept in mm/yr (a)",
          "Shaded Linear - Height Intercept in cm/yr (a)",
          "Shaded Linear - Diam Slope (b)",
          "Shaded Linear - Height Slope (b)",
          "Shaded Linear - Diam Shade Exponent (c)",
          "Shaded Linear - Height Shade Exponent (c)",
          "Size Dep. Logistic - Diam Intercept (a)",
          "Size Dep. Logistic - Height Intercept (a)",
          "Size Dep. Logistic - Diam Slope (b)",
          "Size Dep. Logistic - Height Slope (b)",
          "Size Dep. Logistic - Diam Shape Param 1 (c)",
          "Size Dep. Logistic - Height Shape Param 1 (c)",
          "Size Dep. Logistic - Diam Shape Param 2 (d)",
          "Size Dep. Logistic - Height Shape Param 2 (d)",
          "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
          "Lognormal - Height Growth Increment at Diam 36, in cm/yr (a)",
          "Lognormal - Diam Shape Parameter (b)",
          "Lognormal - Height Shape Parameter (b)",
          "Lognormal - Diam Effect of Shade (c)",
          "Lognormal - Height Effect of Shade (c)",
          "Double resource - Influence of Resource (C)",
          "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Linear Bi-Level - Intercept for High-Light Growth (a)",
          "Linear Bi-Level - Slope for Low-Light Growth (b)",
          "Linear Bi-Level - Intercept for Low-Light Growth (a)",
          "Linear Bi-Level - Slope for High-Light Growth (b)",
          "Lognormal Bi-Level - X0 for Low-Light Growth",
          "Lognormal Bi-Level - Xb for Low-Light Growth",
          "Lognormal Bi-Level - X0 for High-Light Growth",
          "Lognormal Bi-Level - Xb for High-Light Growth",
          "Lognormal Bi-Level - Max Growth in High Light (m)",
          "Lognormal Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Lognormal Bi-Level - Max Growth in Low Light (m)",
          "PR - Height Threshold for Stochastic Growth (m)",
          "PR - \"a\" Parameter for Deterministic Growth",
          "PR - \"b\" Parameter for Deterministic Growth",
          "PR - Mean DBH (cm) for Stochastic Growth",
          "PR - DBH Standard Deviation for Stochastic Growth",
          "PR Storm Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "PR Storm Bi-Level - High-Light \"a\"",
          "PR Storm Bi-Level - High-Light \"b\"",
          "PR Storm Bi-Level - Slope for Low-Light Growth (b)",
          "PR Storm Bi-Level - Intercept for Low-Light Growth (a)",
          "Browsed Asymptotic Diameter Growth (A)",
          "Browsed Slope of Growth Response (S)",
          "Browsed Diameter Exponent"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for growth.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }

  }

  /**
   * Makes sure that nci lambda values are correctly read from parameter
   * file.
   */
  public void testReadParFile() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.6640108, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.71, 0.01);
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex + 1);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.00442797, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.12, 0.01);

      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Makes sure that nci lambda values are managed correctly when species
   * change.
   */
  public void testChangeOfSpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 1"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.6640108, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.71, 0.01);

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex + 1);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.00442797, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.12, 0.01);

      //Change the species
     String[] sNewSpecies = new String[] {
            "Species 2",
            "Species 3",
            "Species 4"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      oGrowth = oManager.GetGrowthBehaviors();

      //Count the total number of NCIs
      int iCount = 0;
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iCount++;
        }
      }
      assertEquals(3, iCount);

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.12, 0.01);

      iIndex++;
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3") &&
                 -1 != oVector.GetDescriptor().indexOf("NCI Lambda"));

      iIndex++;
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 4") &&
                 -1 != oVector.GetDescriptor().indexOf("NCI Lambda"));

      iIndex++;
      if (iIndex < oGrowth.mp_oAllData.size()) {
       oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
       assertTrue(-1 == oVector.GetDescriptor().indexOf("NCI Lambda"));
     }
    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Makes sure that nci lambda values are managed correctly when species
   * are copied.
   */
  public void testCopySpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLFileForSpeciesCopy();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }

      //Verify how the data was read in before species copy
      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(4, oBeh.GetNumberOfCombos());
      SpeciesTypeCombo oCombo;
      for (i = 0; i < oBeh.GetNumberOfCombos(); i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(3, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }

      Float fVal;

      //Max potential growth
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(0);
      assertEquals(1.52, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(1);
      assertEquals(2.5, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(2);
      assertEquals(1.18, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(3);
      assertEquals(1.19, fVal.floatValue(), 0.001);

      //NCI Alpha
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(0);
      assertEquals(2.17, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(1);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(2);
      assertEquals(1.51, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(3);
      assertEquals(1.6, fVal.floatValue(), 0.001);

      //NCI Beta
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(0);
      assertEquals(0.69, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(1);
      assertEquals(0.46, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(2);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(3);
      assertEquals(0.28, fVal.floatValue(), 0.001);

      //Lambdas
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.43, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.48, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.52, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.78, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.33, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.31, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.87, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.74, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.88, fVal.floatValue(), 0.001);

      //Copy the species
      oManager.GetTreePopulation().DoCopySpecies("Western_Redcedar", "Black_Cottonwood");

      oBeh = oGrowth.GetBehaviorByKey("nci");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(5, oBeh.GetNumberOfCombos());
      for (i = 0; i < oBeh.GetNumberOfCombos() - 1; i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(3, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }
      oCombo = oBeh.GetSpeciesTypeCombo(4);
      assertEquals(3, oCombo.GetType());
      assertEquals(7, oCombo.GetSpecies());

      //Max potential growth
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(0);
      assertEquals(1.52, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(1);
      assertEquals(2.5, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(2);
      assertEquals(1.18, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(3);
      assertEquals(1.19, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(7);
      assertEquals(2.5, fVal.floatValue(), 0.001);

      //NCI Alpha
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(0);
      assertEquals(2.17, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(1);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(2);
      assertEquals(1.51, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(3);
      assertEquals(1.6, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(7);
      assertEquals(1.86, fVal.floatValue(), 0.001);

      //NCI Beta
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(0);
      assertEquals(0.69, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(1);
      assertEquals(0.46, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(2);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(3);
      assertEquals(0.28, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(7);
      assertEquals(0.46, fVal.floatValue(), 0.001);

      //Lambdas
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.43, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.48, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.52, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.78, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.74, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.88, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.19, fVal.floatValue(), 0.001);

      System.out.println("Test copy species was successful.");
    }
    catch (ModelException oErr) {
      fail("Test copy species failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests ValidateData().
   */
  public void testValidateData() {
    GUIManager oManager = null;
    String sFileName = null;
    TreePopulation oPop = null;
    try {

      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 2
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 4
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 4.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 5
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 5.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 6
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile6();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 6.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 7
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile7();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 7.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 8
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile8();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 8.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 9
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile9();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 9.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 10
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile10();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 10.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 11
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile11();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 11.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 12
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile12();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 12.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 13
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile13();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 13.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 14
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile14();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 14.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 15
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile15();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 15.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 16
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile16();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 16.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 17
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile17();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 17.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 18
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile18();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 18.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Light threshold not between 0 and 100 for linear bi-level growth
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("linear bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(-20));
      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad light threshold values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Light threshold not between 0 and 100 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelHighLightThreshold.GetValue().add(new Float(-20));
      oGrowth.mp_fLogBiLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad light threshold values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Low-light X0 = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelLoLiteX0.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelLoLiteX0.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad X0 value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //High-light X0 = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelHiLiteX0.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelHiLiteX0.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad X0 value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Low-light Xb = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelLoLiteXb.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelLoLiteXb.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad Xb value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //High-light Xb = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelHiLiteXb.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelHiLiteXb.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad Xb value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //High-light max growth < 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelHiLiteMaxGrwth.GetValue().add(new Float(-20));
      oGrowth.mp_fLogBiLevelHiLiteMaxGrwth.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad max growth value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Low-light max growth < 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelLoLiteMaxGrwth.GetValue().add(new Float(-20));
      oGrowth.mp_fLogBiLevelLoLiteMaxGrwth.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad max growth value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Valid pr storm bi-level growth situation
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      //Enable storms and storm light
      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Storms");
      oBeh.m_bIsEnabled = true;
      oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
      oBeh.m_bIsEnabled = true;

      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(20));
      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message: " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }


    try {
      //Light threshold not between 0 and 100 for pr storm bi-level growth
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      //Enable storms and storm light
      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Storms");
      oBeh.m_bIsEnabled = true;
      oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
      oBeh.m_bIsEnabled = true;

      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(-20));
      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad light threshold values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //No storm behavior with pr storm bi-level growth
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      //Enable storm light
      oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
      oBeh.m_bIsEnabled = true;

      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(20));
      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice lack of storms with pr storm bi-level growth.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //No storm light behavior with pr storm bi-level growth
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      //Enable storms
      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Storms");
      oBeh.m_bIsEnabled = true;

      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(20));
      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice lack of storm light with pr storm bi-level growth.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    System.out.println("Growth ValidateData testing succeeded.");
  }

  /**
   * Writes a valid NCI growth file, Diam with auto height.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  protected String WriteXMLValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid NCI growth file, Diam only.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLValidFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 1.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile1End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 2.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile1End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 3.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile2End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 4.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile2End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 5.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile3End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 6.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile6() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile3End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 7.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile7() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile4End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 8.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile8() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile4End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 9.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile9() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile5End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 10.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile10() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile5End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 11.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile11() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile6End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 12.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile12() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile6End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 13.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile13() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile7End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 14.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile14() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile7End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 15.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile15() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile8End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 16.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile16() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile8End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 17.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile17() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteErrorFile9End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Creates error file 18.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename for the file just written.
   */
  protected String WriteXMLErrorFile18() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteErrorFile9End(oOut);
    oOut.close();
    return sFileName;
  }

  /**
   * Writes the common first part of parameter files.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteValidateFilePart1(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

  /**
   * Writes a behavior list for diameter NCI growth with auto height.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteBehaviorListDiamAuto(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>ncigrowth</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  /**
   * Writes a behavior list for diameter only NCI growth.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteBehaviorListDiamOnly(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>ncigrowth diam only</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  /**
   * Writes NCI max crowding radius parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCIMaxCrowdingRadius(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nmcrVal species=\"Species_1\">10</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Species_2\">10</gr_nmcrVal>");
    oOut.write("</gr_nciMaxCrowdingRadius>");
  }

  /**
   * Writes NCI minimum neighbor DBH parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCIMinNeighborDBH(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciMinNeighborDBH>");
    oOut.write("<gr_nmndVal species=\"Species_1\">10</gr_nmndVal>");
    oOut.write("<gr_nmndVal species=\"Species_2\">10</gr_nmndVal>");
    oOut.write("</gr_nciMinNeighborDBH>");
  }

  /**
   * Writes NCI neighbor DBH effect parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCINeighborDBHEffect(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciAlpha>");
    oOut.write("<gr_naVal species=\"Species_1\">2.17031683</gr_naVal>");
    oOut.write("<gr_naVal species=\"Species_2\">2.81</gr_naVal>");
    oOut.write("</gr_nciAlpha>");
  }

  /**
   * Writes NCI neighbor distance effect parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCINeighborDistEffect(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciBeta>");
    oOut.write("<gr_nbVal species=\"Species_1\">0.699942</gr_nbVal>");
    oOut.write("<gr_nbVal species=\"Species_2\">0.5</gr_nbVal>");
    oOut.write("</gr_nciBeta>");
  }

  /**
   * Writes NCI max potential growth parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCIMaxPotentialGrowth(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nmpgVal species=\"Species_1\">10.5258749</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Species_2\">15.31</gr_nmpgVal>");
    oOut.write("</gr_nciMaxPotentialGrowth>");
  }

  /**
   * Writes NCI size sensitivity parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCISizeSensNCI(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciSizeSensNCI>");
    oOut.write("<gr_nssnVal species=\"Species_1\">0.43</gr_nssnVal>");
    oOut.write("<gr_nssnVal species=\"Species_2\">0.36</gr_nssnVal>");
    oOut.write("</gr_nciSizeSensNCI>");
  }

  /**
   * Writes NCI size effect mode parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCISizeEffectMode(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciSizeEffectMode>");
    oOut.write("<gr_nsemVal species=\"Species_1\">34.24638994</gr_nsemVal>");
    oOut.write("<gr_nsemVal species=\"Species_2\">122.23</gr_nsemVal>");
    oOut.write("</gr_nciSizeEffectMode>");
  }

  /**
   * Writes NCI size effect variance parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCISizeEffectVar(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciSizeEffectVariance>");
    oOut.write("<gr_nsevVal species=\"Species_1\">5</gr_nsevVal>");
    oOut.write("<gr_nsevVal species=\"Species_2\">2.36</gr_nsevVal>");
    oOut.write("</gr_nciSizeEffectVariance>");
  }

  /**
   * Writes NCI slope parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCISlope(FileWriter oOut) throws java.io.IOException {
    oOut.write("<gr_nciCrowdingSlope>");
    oOut.write("<gr_ncslVal species=\"Species_1\">1.98</gr_ncslVal>");
    oOut.write("<gr_ncslVal species=\"Species_2\">4.57</gr_ncslVal>");
    oOut.write("</gr_nciCrowdingSlope>");
  }

  /**
   * Writes NCI steepness parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCISteepness(FileWriter oOut) throws java.io.IOException {
    oOut.write("<gr_nciCrowdingSteepness>");
    oOut.write("<gr_ncstVal species=\"Species_1\">1.63</gr_ncstVal>");
    oOut.write("<gr_ncstVal species=\"Species_2\">1.26</gr_ncstVal>");
    oOut.write("</gr_nciCrowdingSteepness>");
  }

  /**
   * Writes NCI storm effect medium damage parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCIStormEffMedDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciStormEffMedDmg>");
    oOut.write("<gr_nsemdVal species=\"Species_1\">0.89</gr_nsemdVal>");
    oOut.write("<gr_nsemdVal species=\"Species_2\">0.9</gr_nsemdVal>");
    oOut.write("</gr_nciStormEffMedDmg>");
  }

  /**
   * Writes NCI storm effect full damage parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCIStormEffFullDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciStormEffFullDmg>");
    oOut.write("<gr_nsefdVal species=\"Species_1\">0.42</gr_nsefdVal>");
    oOut.write("<gr_nsefdVal species=\"Species_2\">0.34</gr_nsefdVal>");
    oOut.write("</gr_nciStormEffFullDmg>");
  }

  /**
   * Writes NCI neighbor storm effect medium damage parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCINeighStormEffMedDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciNeighStormEffMedDmg>");
    oOut.write("<gr_nnsemdVal species=\"Species_1\">0.89</gr_nnsemdVal>");
    oOut.write("<gr_nnsemdVal species=\"Species_2\">0.9</gr_nnsemdVal>");
    oOut.write("</gr_nciNeighStormEffMedDmg>");
  }

  /**
   * Writes NCI neighbor storm effect full damage parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCINeighStormEffFullDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciNeighStormEffFullDmg>");
    oOut.write("<gr_nnsefdVal species=\"Species_1\">0.42</gr_nnsefdVal>");
    oOut.write("<gr_nnsefdVal species=\"Species_2\">0.34</gr_nnsefdVal>");
    oOut.write("</gr_nciNeighStormEffFullDmg>");
  }

  /**
   * Writes NCI shading coefficient parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCIShadingCoefficient(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<gr_nciShadingCoefficient>");
    oOut.write("<gr_nscVal species=\"Species_1\">0.42</gr_nscVal>");
    oOut.write("<gr_nscVal species=\"Species_2\">0.34</gr_nscVal>");
    oOut.write("</gr_nciShadingCoefficient>");
  }

  /**
   * Writes NCI shading exponent parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCIShadingExponent(FileWriter oOut) throws java.io.
        IOException {
      oOut.write("<gr_nciShadingExponent>");
      oOut.write("<gr_nseVal species=\"Species_1\">0.42</gr_nseVal>");
      oOut.write("<gr_nseVal species=\"Species_2\">0.34</gr_nseVal>");
      oOut.write("</gr_nciShadingExponent>");
    }

  /**
   * Writes NCI lambdas parameter.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteNCILambdas(FileWriter oOut) throws java.io.IOException {
    oOut.write("<gr_nciSpecies_1NeighborLambda>");
    oOut.write("<gr_nlVal species=\"Species_1\">0.6640108</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Species_2\">0.71</gr_nlVal>");
    oOut.write("</gr_nciSpecies_1NeighborLambda>");
    oOut.write("<gr_nciSpecies_2NeighborLambda>");
    oOut.write("<gr_nlVal species=\"Species_1\">0.00442797</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Species_2\">0.12</gr_nlVal>");
    oOut.write("</gr_nciSpecies_2NeighborLambda>");

  }

  /**
   * Writes the end for a valid parameter file.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteValidFileEnd(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where max radius of neighbor Effects is < 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile1End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    oOut.write("<gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nmcrVal species=\"Species_1\">10</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Species_2\">-10</gr_nmcrVal>");
    oOut.write("</gr_nciMaxCrowdingRadius>");
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCIShadingExponent(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where max growth for a species is < 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile2End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    oOut.write("<gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nmpgVal species=\"Species_1\">15.31</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Species_2\">-15.31</gr_nmpgVal>");
    oOut.write("</gr_nciMaxPotentialGrowth>");
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where Xb for a species = 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile3End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    oOut.write("<gr_nciSizeEffectVariance>");
    oOut.write("<gr_nsevVal species=\"Species_1\">0</gr_nsevVal>");
    oOut.write("<gr_nsevVal species=\"Species_2\">5</gr_nsevVal>");
    oOut.write("</gr_nciSizeEffectVariance>");
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where X0 for a species = 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile4End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    oOut.write("<gr_nciSizeEffectMode>");
    oOut.write("<gr_nsemVal species=\"Species_1\">0</gr_nsemVal>");
    oOut.write("<gr_nsemVal species=\"Species_2\">34.2</gr_nsemVal>");
    oOut.write("</gr_nciSizeEffectMode>");
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where med. damage eta for a species is not a
   * proportion.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile5End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    oOut.write("<gr_nciNeighStormEffMedDmg>");
    oOut.write("<gr_nnsemdVal species=\"Species_1\">-0.33</gr_nnsemdVal>");
    oOut.write("<gr_nnsemdVal species=\"Species_2\">0.33</gr_nnsemdVal>");
    oOut.write("</gr_nciNeighStormEffMedDmg>");
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where full damage eta for a species is not a
   * proportion.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile6End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    oOut.write("<gr_nciNeighStormEffFullDmg>");
    oOut.write("<gr_nnsefdVal species=\"Species_1\">-0.54</gr_nnsefdVal>");
    oOut.write("<gr_nnsefdVal species=\"Species_2\">0.54</gr_nnsefdVal>");
    oOut.write("</gr_nciNeighStormEffFullDmg>");
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where med. damage storm Effect for a species is
   * not a proportion.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile7End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    oOut.write("<gr_nciStormEffMedDmg>");
    oOut.write("<gr_nsemdVal species=\"Species_1\">0.9</gr_nsemdVal>");
    oOut.write("<gr_nsemdVal species=\"Species_2\">1.9</gr_nsemdVal>");
    oOut.write("</gr_nciStormEffMedDmg>");
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where full damage storm Effect for a species is
   * not a proportion.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile8End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    oOut.write("<gr_nciStormEffFullDmg>");
    oOut.write("<gr_nsefdVal species=\"Species_1\">-0.42</gr_nsefdVal>");
    oOut.write("<gr_nsefdVal species=\"Species_2\">0.42</gr_nsefdVal>");
    oOut.write("</gr_nciStormEffFullDmg>");
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    WriteNCIMinNeighborDBH(oOut);
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes an error condition where minimum neighbor DBH is less than 0.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteErrorFile9End(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxPotentialGrowth(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCILambdas(oOut);
    WriteNCIShadingCoefficient(oOut);
    WriteNCIShadingExponent(oOut);
    oOut.write("<gr_nciMinNeighborDBH>");
    oOut.write("<gr_nmndVal species=\"Species_1\">10</gr_nmndVal>");
    oOut.write("<gr_nmndVal species=\"Species_2\">-10</gr_nmndVal>");
    oOut.write("</gr_nciMinNeighborDBH>");
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
  }

  /**
   * Writes a file with no growth settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>August 14, 2006: Created (LEM)
   */
  protected String WriteNoGrowthXMLFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>densityselfthinning</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  protected String WriteXMLFileForSpeciesCopy() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Western_Hemlock\" />");
    oOut.write("<tr_species speciesName=\"Western_Redcedar\" />");
    oOut.write("<tr_species speciesName=\"Amabilis_Fir\" />");
    oOut.write("<tr_species speciesName=\"Subalpine_Fir\" />");
    oOut.write("<tr_species speciesName=\"Hybrid_Spruce\" />");
    oOut.write("<tr_species speciesName=\"Lodgepole_Pine\" />");
    oOut.write("<tr_species speciesName=\"Trembling_Aspen\" />");
    oOut.write("<tr_species speciesName=\"Black_Cottonwood\" />");
    oOut.write("<tr_species speciesName=\"Paper_Birch\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>ncigrowth</behaviorName>");
    oOut.write("<version>2.1</version>");
    oOut.write("<applyTo species=\"Western_Hemlock\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Western_Redcedar\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Amabilis_Fir\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Subalpine_Fir\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<growth>");
    oOut.write("<gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nmpgVal species=\"Western_Hemlock\">1.52</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Western_Redcedar\">2.5</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Amabilis_Fir\">1.18</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Subalpine_Fir\">1.19</gr_nmpgVal>");
    oOut.write("</gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nmcrVal species=\"Western_Hemlock\">10.0</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Western_Redcedar\">10.0</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Amabilis_Fir\">10.0</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Subalpine_Fir\">10.0</gr_nmcrVal>");
    oOut.write("</gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nciAlpha>");
    oOut.write("<gr_naVal species=\"Western_Hemlock\">2.17</gr_naVal>");
    oOut.write("<gr_naVal species=\"Western_Redcedar\">1.86</gr_naVal>");
    oOut.write("<gr_naVal species=\"Amabilis_Fir\">1.51</gr_naVal>");
    oOut.write("<gr_naVal species=\"Subalpine_Fir\">1.6</gr_naVal>");
    oOut.write("</gr_nciAlpha>");
    oOut.write("<gr_nciBeta>");
    oOut.write("<gr_nbVal species=\"Western_Hemlock\">0.69</gr_nbVal>");
    oOut.write("<gr_nbVal species=\"Western_Redcedar\">0.46</gr_nbVal>");
    oOut.write("<gr_nbVal species=\"Amabilis_Fir\">0.18</gr_nbVal>");
    oOut.write("<gr_nbVal species=\"Subalpine_Fir\">0.28</gr_nbVal>");
    oOut.write("</gr_nciBeta>");
    oOut.write("<gr_nciWestern_HemlockNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.66</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.37</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.40</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.43</gr_nlVal>");
    oOut.write("</gr_nciWestern_HemlockNeighborLambda>");
    oOut.write("<gr_nciWestern_RedcedarNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.44</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.49</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">4.58E-4</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">5.58E-4</gr_nlVal>");
    oOut.write("</gr_nciWestern_RedcedarNeighborLambda>");
    oOut.write("<gr_nciAmabilis_FirNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.41</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">6.21E-4</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.21</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.25</gr_nlVal>");
    oOut.write("</gr_nciAmabilis_FirNeighborLambda>");
    oOut.write("<gr_nciSubalpine_FirNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.41</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">6.21E-4</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.21</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.18</gr_nlVal>");
    oOut.write("</gr_nciSubalpine_FirNeighborLambda>");
    oOut.write("<gr_nciHybrid_SpruceNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.41</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">6.21E-4</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.27</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.22</gr_nlVal>");
    oOut.write("</gr_nciHybrid_SpruceNeighborLambda>");
    oOut.write("<gr_nciLodgepole_PineNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.48</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">6.21</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.52</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.18</gr_nlVal>");
    oOut.write("</gr_nciLodgepole_PineNeighborLambda>");
    oOut.write("<gr_nciTrembling_AspenNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.38</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.61</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.38</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.78</gr_nlVal>");
    oOut.write("</gr_nciTrembling_AspenNeighborLambda>");
    oOut.write("<gr_nciBlack_CottonwoodNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.33</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.66</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.31</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.87</gr_nlVal>");
    oOut.write("</gr_nciBlack_CottonwoodNeighborLambda>");
    oOut.write("<gr_nciPaper_BirchNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.74</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.19</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.14</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.88</gr_nlVal>");
    oOut.write("</gr_nciPaper_BirchNeighborLambda>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}
