package javawrapper;

import java.io.FileWriter;
import java.io.File;

/**
 * Tests the Storms behavior of the DisturbanceBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestStormBehaviors
    extends ModelTestCase {

  /**
   * Makes sure the correct options are displayed in parameters.
   */
  public void testFormatDataForDisplay() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no disturbance behaviors are enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      TreePopulation oPop = oManager.GetTreePopulation();
      DisturbanceBehaviors oDisturbance = oManager.GetDisturbanceBehaviors();
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      //Case:  only storms is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Storms");
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Return Interval for Severity Storm Class 0 - 0.1",
          "Return Interval for Severity Storm Class 0.1 - 0.2",
          "Return Interval for Severity Storm Class 0.2 - 0.3",
          "Return Interval for Severity Storm Class 0.3 - 0.4",
          "Return Interval for Severity Storm Class 0.4 - 0.5",
          "Return Interval for Severity Storm Class 0.5 - 0.6",
          "Return Interval for Severity Storm Class 0.6 - 0.7",
          "Return Interval for Severity Storm Class 0.7 - 0.8",
          "Return Interval for Severity Storm Class 0.8 - 0.9",
          "Return Interval for Severity Storm Class 0.9 - 1.0",
          "Storm - Sea Surface Temperature Cyclicity Period (Years)",
          "Storm - Storm Cyclicity Sine Curve d",
          "Storm - Storm Cyclicity Sine Curve f",
          "Storm - Storm Cyclicity Sine Curve g",
          "Storm - Storm Cyclicity Trend Function Slope (m)",
          "Storm - Storm Cyclicity Trend Function Intercept (i)",
          "Standard Deviation (lognormal or normal)",
          "Plot Storm Susceptibility Pattern",
          "Storm Damage Application",
          "Stochastic Pattern Damage Distribution"};
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      //Case: only storm damage applier is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Storm Damage Applier");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Minimum DBH for Storm Damage, in cm",
          "Storm Damage Intercept (a) for Medium Damage",
          "Storm Damage Intercept (a) for Heavy Damage",
          "Storm Intensity Coefficient (b)",
          "Storm DBH Coefficient (d)",
          "Number of Years Damaged Trees Take to Heal"};
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      //Case: only storm killer is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Storm Killer");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Minimum DBH for Storm Damage, in cm",
          "Storm Medium Damage Survival Prob Intercept (a)",
          "Storm Medium Damage Survival Prob DBH Coeff. (b)",
          "Storm Heavy Damage Survival Prob Intercept (a)",
          "Storm Heavy Damage Survival Prob DBH Coeff. (b)",
          "Storm - Prop. Heavy Damage Dead Trees that Tip Up",
          "Number of Years Storm-Damaged Snags Last"};
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      //Case: only windstorm is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Windstorm");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Windstorm - Minimum DBH for Windstorm Mortality",
          "Windstorm - Mortality Intercept (a)",
          "Windstorm - Storm Intensity Coefficient (c)",
          "Windstorm - DBH Exponent (b)",
          "Windstorm - Severity for 1 Year Return Interval Storm",
          "Windstorm - Severity for 5 Year Return Interval Storm",
          "Windstorm - Severity for 10 Year Return Interval Storm",
          "Windstorm - Severity for 20 Year Return Interval Storm",
          "Windstorm - Severity for 40 Year Return Interval Storm",
          "Windstorm - Severity for 80 Year Return Interval Storm",
          "Windstorm - Severity for 160 Year Return Interval Storm",
          "Windstorm - Severity for 320 Year Return Interval Storm",
          "Windstorm - Severity for 640 Year Return Interval Storm",
          "Windstorm - Severity for 1280 Year Return Interval Storm",
          "Windstorm - Severity for 2560 Year Return Interval Storm",
          "Windstorm - Timestep to Start Storms",
          "Windstorm - Sea Surface Temperature Cyclicity Period (Years)",
          "Windstorm - Storm Cyclicity Sine Curve d",
          "Windstorm - Storm Cyclicity Sine Curve f",
          "Windstorm - Storm Cyclicity Sine Curve g",
          "Windstorm - Storm Cyclicity Trend Function Slope (m)",
          "Windstorm - Storm Cyclicity Trend Function Intercept (i)"};
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  Only Harvest is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Harvest");
      oBeh.m_bIsEnabled = true;
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      //Case:  Only Natural Disturbance is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Episodic Mortality");
      oBeh.m_bIsEnabled = true;
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));

      //Case:  Only Harvest Interface is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Harvest Interface");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.m_bIsEnabled = true;
      assertEquals(null, oDisturbance.FormatDataForDisplay(oPop));


      //Case:  only selection harvest is enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      oBeh = oDisturbance.GetBehaviorByKey("Selection Harvest");
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Selection Harvest Cut Range 1 Lower DBH (cm)",
          "Selection Harvest Cut Range 1 Upper DBH (cm)",
          "Selection Harvest Cut Range 1 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 2 Lower DBH (cm)",
          "Selection Harvest Cut Range 2 Upper DBH (cm)",
          "Selection Harvest Cut Range 2 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 3 Lower DBH (cm)",
          "Selection Harvest Cut Range 3 Upper DBH (cm)",
          "Selection Harvest Cut Range 3 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 4 Lower DBH (cm)",
          "Selection Harvest Cut Range 4 Upper DBH (cm)",
          "Selection Harvest Cut Range 4 Target Basal Area (m2/ha)",
          "Selection Harvest Initial Age"};
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  all are enabled.
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oDisturbance = oManager.GetDisturbanceBehaviors();
      int i;
      for (i = 0; i < oDisturbance.mp_oChildBehaviors.length; i++) {
        oDisturbance.mp_oChildBehaviors[i].m_bIsEnabled = true;
        if (oDisturbance.mp_oChildBehaviors[i].GetKey().equals("Storm Damage Applier") ||
            oDisturbance.mp_oChildBehaviors[i].GetKey().equals("Storm Killer") ||
            oDisturbance.mp_oChildBehaviors[i].GetKey().equals("Windstorm") ||
            oDisturbance.mp_oChildBehaviors[i].GetKey().equals("Random Browse") ||
            oDisturbance.mp_oChildBehaviors[i].GetKey().equals("Competition Harvest")) {
          oDisturbance.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(0, 3, oPop));
        }
      }
      p_sExpected = new String[] {
          "Return Interval for Severity Storm Class 0 - 0.1",
          "Return Interval for Severity Storm Class 0.1 - 0.2",
          "Return Interval for Severity Storm Class 0.2 - 0.3",
          "Return Interval for Severity Storm Class 0.3 - 0.4",
          "Return Interval for Severity Storm Class 0.4 - 0.5",
          "Return Interval for Severity Storm Class 0.5 - 0.6",
          "Return Interval for Severity Storm Class 0.6 - 0.7",
          "Return Interval for Severity Storm Class 0.7 - 0.8",
          "Return Interval for Severity Storm Class 0.8 - 0.9",
          "Return Interval for Severity Storm Class 0.9 - 1.0",
          "Storm - Sea Surface Temperature Cyclicity Period (Years)",
          "Storm - Storm Cyclicity Sine Curve d",
          "Storm - Storm Cyclicity Sine Curve f",
          "Storm - Storm Cyclicity Sine Curve g",
          "Storm - Storm Cyclicity Trend Function Slope (m)",
          "Storm - Storm Cyclicity Trend Function Intercept (i)",
          "Standard Deviation (lognormal or normal)",
          "Plot Storm Susceptibility Pattern",
          "Storm Damage Application",
          "Stochastic Pattern Damage Distribution",
          "Minimum DBH for Storm Damage, in cm",
          "Storm Damage Intercept (a) for Medium Damage",
          "Storm Damage Intercept (a) for Heavy Damage",
          "Storm Intensity Coefficient (b)",
          "Storm DBH Coefficient (d)",
          "Number of Years Damaged Trees Take to Heal",
          "Storm Medium Damage Survival Prob Intercept (a)",
          "Storm Medium Damage Survival Prob DBH Coeff. (b)",
          "Storm Heavy Damage Survival Prob Intercept (a)",
          "Storm Heavy Damage Survival Prob DBH Coeff. (b)",
          "Storm - Prop. Heavy Damage Dead Trees that Tip Up",
          "Number of Years Storm-Damaged Snags Last",
          "Random Browse - Annual Browse Probability (0-1)",
          "Random Browse - Browse Probability Standard Deviation",
          "Random Browse - Probability PDF",
          "Selection Harvest Cut Range 1 Lower DBH (cm)",
          "Selection Harvest Cut Range 1 Upper DBH (cm)",
          "Selection Harvest Cut Range 1 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 2 Lower DBH (cm)",
          "Selection Harvest Cut Range 2 Upper DBH (cm)",
          "Selection Harvest Cut Range 2 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 3 Lower DBH (cm)",
          "Selection Harvest Cut Range 3 Upper DBH (cm)",
          "Selection Harvest Cut Range 3 Target Basal Area (m2/ha)",
          "Selection Harvest Cut Range 4 Lower DBH (cm)",
          "Selection Harvest Cut Range 4 Upper DBH (cm)",
          "Selection Harvest Cut Range 4 Target Basal Area (m2/ha)",
          "Selection Harvest Initial Age",
          "Windstorm - Minimum DBH for Windstorm Mortality",
          "Windstorm - Mortality Intercept (a)",
          "Windstorm - Storm Intensity Coefficient (c)",
          "Windstorm - DBH Exponent (b)",
          "Windstorm - Severity for 1 Year Return Interval Storm",
          "Windstorm - Severity for 5 Year Return Interval Storm",
          "Windstorm - Severity for 10 Year Return Interval Storm",
          "Windstorm - Severity for 20 Year Return Interval Storm",
          "Windstorm - Severity for 40 Year Return Interval Storm",
          "Windstorm - Severity for 80 Year Return Interval Storm",
          "Windstorm - Severity for 160 Year Return Interval Storm",
          "Windstorm - Severity for 320 Year Return Interval Storm",
          "Windstorm - Severity for 640 Year Return Interval Storm",
          "Windstorm - Severity for 1280 Year Return Interval Storm",
          "Windstorm - Severity for 2560 Year Return Interval Storm",
          "Windstorm - Timestep to Start Storms",
          "Windstorm - Sea Surface Temperature Cyclicity Period (Years)",
          "Windstorm - Storm Cyclicity Sine Curve d",
          "Windstorm - Storm Cyclicity Sine Curve f",
          "Windstorm - Storm Cyclicity Sine Curve g",
          "Windstorm - Storm Cyclicity Trend Function Slope (m)",
          "Windstorm - Storm Cyclicity Trend Function Intercept (i)",
          "Competition Harvest: Max Radius of Competitive Effects (m)",
          "Competition Harvest: Amount of Harvest Per Species (0 - 1)",
          "Competition Harvest: DBH Effect of Targets (alpha)",
          "Competition Harvest: Distance Effect of Targets (beta)",
          "Competition Harvest: Size Sensitivity (gamma)",
          "Competition Harvest: C",
          "Competition Harvest: D",
          "Competition Harvest: Target DBH Divisor",
          "Competition Harvest: Minimum DBH to Harvest",
          "Competition Harvest: Maximum DBH to Harvest",
          "Competition Harvest: Harvest Type",
          "Competition Harvest: Amount to Harvest",
          "Competition Harvest: Min Years Between Fixed BA Harvests",
          "Competition Harvest: Fixed Interval Harvest Interval (yr)",
          "Competition Harvest: Fixed BA Harvest Threshold (m2/ha)",
          "Competition Harvest: Filename for List of Harvested Trees",
          "Competition Harvest: Species 1 Target Lambda",
          };
      TestTable(oDisturbance.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for disturbance.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests parameter file reading.  This makes sure that all values for
   * storms make it where they are supposed to.
   * @throws ModelException if a test fails or a parameter file cannot be
   * written or parsed.
   */
  public void testReadParFile() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //File 1 - deterministic uniform damage pattern
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      assertEquals(oDist.m_fClass1RtrnInt.GetValue(), 0.1, 0.001);
      assertEquals(oDist.m_fClass2RtrnInt.GetValue(), 0.2, 0.001);
      assertEquals(oDist.m_fClass3RtrnInt.GetValue(), 0.3, 0.001);
      assertEquals(oDist.m_fClass4RtrnInt.GetValue(), 0.4, 0.001);
      assertEquals(oDist.m_fClass5RtrnInt.GetValue(), 0.5, 0.001);
      assertEquals(oDist.m_fClass6RtrnInt.GetValue(), 0.6, 0.001);
      assertEquals(oDist.m_fClass7RtrnInt.GetValue(), 0.7, 0.001);
      assertEquals(oDist.m_fClass8RtrnInt.GetValue(), 0.8, 0.001);
      assertEquals(oDist.m_fClass9RtrnInt.GetValue(), 0.9, 0.001);
      assertEquals(oDist.m_fClass10RtrnInt.GetValue(), 1.0, 0.001);
      assertTrue(oDist.m_iStochasticity.GetStringValue().equalsIgnoreCase(
          "deterministic"));
      assertTrue(oDist.m_iSusceptibility.GetStringValue().equalsIgnoreCase(
          "uniform"));

      assertEquals( ( (Float) oDist.mp_fMinStormDamageDBH.GetValue().elementAt(0)).
                   floatValue(), 5, 0.001);
      assertEquals( ( (Float) oDist.mp_fMinStormDamageDBH.GetValue().elementAt(1)).
                   floatValue(), 6, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptMed.GetValue().
                      elementAt(0)).floatValue(), 2.1789601, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptMed.GetValue().
                      elementAt(1)).floatValue(), 2.772772, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptFull.GetValue().
                     elementAt(0)).floatValue(), 2.6334169, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptFull.GetValue().
                     elementAt(1)).floatValue(), 4.9189817, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmIntensityCoeff.GetValue().
                      elementAt(0)).floatValue(), -0.2545352, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmIntensityCoeff.GetValue().
                      elementAt(1)).floatValue(), -1.936994, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmDBHCoeff.GetValue().elementAt(0)).
                   floatValue(), 0.8280319, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmDBHCoeff.GetValue().elementAt(1)).
                   floatValue(), 0.2411269, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmMedDmgSurvProbA.GetValue().
                      elementAt(0)).floatValue(), 0.1, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmMedDmgSurvProbA.GetValue().
                      elementAt(1)).floatValue(), 0.2, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmMedDmgSurvProbB.GetValue().
                      elementAt(0)).floatValue(), 0.3, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmMedDmgSurvProbB.GetValue().
                      elementAt(1)).floatValue(), 0.4, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmFullDmgSurvProbA.GetValue().
                      elementAt(0)).floatValue(), -0.1, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmFullDmgSurvProbA.GetValue().
                      elementAt(1)).floatValue(), -0.2, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmFullDmgSurvProbB.GetValue().
                      elementAt(0)).floatValue(), -0.3, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmFullDmgSurvProbB.GetValue().
                      elementAt(1)).floatValue(), -0.4, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmPropFullTipUp.GetValue().elementAt(0)).
                   floatValue(), 0.5, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmPropFullTipUp.GetValue().elementAt(1)).
                   floatValue(), 0.6, 0.001);

      assertEquals(4, oDist.m_iNumYearsToHeal.GetValue());
      assertEquals(3, oDist.m_iNumSnagYears.GetValue());

      //Cyclicity defaults
      assertEquals(oDist.m_fStormSSTPeriod.GetValue(), 1, 0.001);
      assertEquals(oDist.m_fStormSineD.GetValue(), 0, 0.001);
      assertEquals(oDist.m_fStormSineF.GetValue(), 1, 0.001);
      assertEquals(oDist.m_fStormSineG.GetValue(), 1, 0.001);
      assertEquals(oDist.m_fStormTrendSlopeM.GetValue(), 0, 0.001);
      assertEquals(oDist.m_fStormTrendInterceptI.GetValue(), 1, 0.001);
      

      new File(sFileName).delete();

      //File 2 - stochastic uniform damage pattern - skip the stuff we already
      //read correctly
      sFileName = WriteXMLValidFile2();
      oManager.InputXMLParameterFile(sFileName);
      oDist = oManager.GetDisturbanceBehaviors();
      assertTrue(oDist.m_iStochasticity.GetStringValue().equalsIgnoreCase(
          "stochastic"));
      assertTrue(oDist.m_iSusceptibility.GetStringValue().equalsIgnoreCase(
          "uniform"));
      assertTrue(oDist.m_iProbDistFunc.GetStringValue().equalsIgnoreCase(
          "lognormal"));
      assertEquals(oDist.m_fStdDev.GetValue(), 0.3, 0.001);
      assertEquals(oDist.m_fStormSSTPeriod.GetValue(), 50, 0.001);
      assertEquals(oDist.m_fStormSineD.GetValue(), 1, 0.001);
      assertEquals(oDist.m_fStormSineF.GetValue(), 2, 0.001);
      assertEquals(oDist.m_fStormSineG.GetValue(), 3, 0.001);
      assertEquals(oDist.m_fStormTrendSlopeM.GetValue(), 4, 0.001);
      assertEquals(oDist.m_fStormTrendInterceptI.GetValue(), -4, 0.001);
      
      new File(sFileName).delete();

      //File 3 - stochastic uniform damage pattern - skip the stuff we already
      //read correctly
      sFileName = WriteXMLValidFile3();
      oManager.InputXMLParameterFile(sFileName);
      oDist = oManager.GetDisturbanceBehaviors();
      assertTrue(oDist.m_iStochasticity.GetStringValue().equalsIgnoreCase(
          "stochastic"));
      assertTrue(oDist.m_iSusceptibility.GetStringValue().equalsIgnoreCase(
          "uniform"));
      assertTrue(oDist.m_iProbDistFunc.GetStringValue().equalsIgnoreCase(
          "normal"));
      new File(sFileName).delete();

      //File 4 - windstorm parameters
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      oDist = oManager.GetDisturbanceBehaviors();
      assertEquals(oDist.m_fWindstormReturnInt1Severity.GetValue(), 0.1, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt5Severity.GetValue(), 0.11, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt10Severity.GetValue(), 0.12, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt20Severity.GetValue(), 0.13, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt40Severity.GetValue(), 0.14, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt80Severity.GetValue(), 0.15, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt160Severity.GetValue(), 0.16, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt320Severity.GetValue(), 0.17, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt640Severity.GetValue(), 0.18, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt1280Severity.GetValue(), 0.19, 0.001);
      assertEquals(oDist.m_fWindstormReturnInt2560Severity.GetValue(), 0.20, 0.001);
      assertEquals(oDist.m_iWindstormTimestepToStartStorms.GetValue(), 1, 0.001);
      assertEquals(oDist.m_fWindstormSSTPeriod.GetValue(), 50, 0.001);
      assertEquals(oDist.m_fWindstormSineD.GetValue(), 0, 0.001);
      assertEquals(oDist.m_fWindstormSineF.GetValue(), 1, 0.001);
      assertEquals(oDist.m_fWindstormSineG.GetValue(), 2, 0.001);
      assertEquals(oDist.m_fWindstormTrendSlopeM.GetValue(), 3, 0.001);
      assertEquals(oDist.m_fWindstormTrendInterceptI.GetValue(), -4, 0.001);
      assertEquals( ( (Float) oDist.mp_fWindstormMinDBH.GetValue().
                      elementAt(0)).floatValue(), 5, 0.001);
      assertEquals( ( (Float) oDist.mp_fWindstormMinDBH.GetValue().
                      elementAt(1)).floatValue(), 6, 0.001);

      assertEquals( ( (Float) oDist.mp_fWindstormMortInterceptA.GetValue().
                      elementAt(0)).floatValue(), -2.261, 0.001);
      assertEquals( ( (Float) oDist.mp_fWindstormMortInterceptA.GetValue().
                      elementAt(1)).floatValue(), -1.96, 0.001);

      assertEquals( ( (Float) oDist.mp_fWindstormDBHExpB.GetValue().
                      elementAt(0)).floatValue(), 0.426, 0.001);
      assertEquals( ( (Float) oDist.mp_fWindstormDBHExpB.GetValue().
                      elementAt(1)).floatValue(), 0.679, 0.001);

      assertEquals( ( (Float) oDist.mp_fWindstormStmIntensCoeffC.GetValue().
                      elementAt(0)).floatValue(), 1.14, 0.001);
      assertEquals( ( (Float) oDist.mp_fWindstormStmIntensCoeffC.GetValue().
                      elementAt(1)).floatValue(), 0.19, 0.001);
    }
    catch (ModelException oErr) {
      fail("Storm validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests backwards compatible parameter file reading.
   * @throws ModelException if a test fails or a parameter file cannot be
   * written or parsed.
   */
  public void testParFileBackwardsCompatibility() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = Write6_05XMLFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();

      //Verify that the storm behaviors are all enabled
      assertTrue(oDist.GetBehaviorByKey("Storms").m_bIsEnabled);
      assertTrue(oDist.GetBehaviorByKey("Storm Damage Applier").m_bIsEnabled);
      assertTrue(oDist.GetBehaviorByKey("Storm Killer").m_bIsEnabled);
      assertEquals(2, oDist.GetBehaviorByKey("Storm Damage Applier").GetNumberOfCombos());
      assertEquals(2, oDist.GetBehaviorByKey("Storm Killer").GetNumberOfCombos());

      //Verify the parameters
      assertEquals(oDist.m_fClass1RtrnInt.GetValue(), 0.1, 0.001);
      assertEquals(oDist.m_fClass2RtrnInt.GetValue(), 0.2, 0.001);
      assertEquals(oDist.m_fClass3RtrnInt.GetValue(), 0.3, 0.001);
      assertEquals(oDist.m_fClass4RtrnInt.GetValue(), 0.4, 0.001);
      assertEquals(oDist.m_fClass5RtrnInt.GetValue(), 0.5, 0.001);
      assertEquals(oDist.m_fClass6RtrnInt.GetValue(), 0.6, 0.001);
      assertEquals(oDist.m_fClass7RtrnInt.GetValue(), 0.7, 0.001);
      assertEquals(oDist.m_fClass8RtrnInt.GetValue(), 0.8, 0.001);
      assertEquals(oDist.m_fClass9RtrnInt.GetValue(), 0.9, 0.001);
      assertEquals(oDist.m_fClass10RtrnInt.GetValue(), 1.0, 0.001);
      assertTrue(oDist.m_iStochasticity.GetStringValue().equalsIgnoreCase(
          "deterministic"));
      assertTrue(oDist.m_iSusceptibility.GetStringValue().equalsIgnoreCase(
          "uniform"));

      assertEquals( ( (Float) oDist.mp_fMinStormDamageDBH.GetValue().elementAt(0)).
                   floatValue(), 5, 0.001);
      assertEquals( ( (Float) oDist.mp_fMinStormDamageDBH.GetValue().elementAt(1)).
                   floatValue(), 6, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptMed.GetValue().
                      elementAt(0)).floatValue(), 2.1789601, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptMed.GetValue().
                      elementAt(1)).floatValue(), 2.772772, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptFull.GetValue().
                     elementAt(0)).floatValue(), 2.6334169, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmDmgInterceptFull.GetValue().
                     elementAt(1)).floatValue(), 4.9189817, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmIntensityCoeff.GetValue().
                      elementAt(0)).floatValue(), -0.2545352, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmIntensityCoeff.GetValue().
                      elementAt(1)).floatValue(), -1.936994, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmDBHCoeff.GetValue().elementAt(0)).
                   floatValue(), 0.8280319, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmDBHCoeff.GetValue().elementAt(1)).
                   floatValue(), 0.2411269, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmMedDmgSurvProbA.GetValue().
                     elementAt(0)).floatValue(), 2.19722, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmMedDmgSurvProbA.GetValue().
                      elementAt(1)).floatValue(), 1.38629, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmMedDmgSurvProbB.GetValue().
                     elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmMedDmgSurvProbB.GetValue().
                      elementAt(1)).floatValue(), 0, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmFullDmgSurvProbA.GetValue().
                     elementAt(0)).floatValue(), 0.847298, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmFullDmgSurvProbA.GetValue().
                      elementAt(1)).floatValue(), 10, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmFullDmgSurvProbB.GetValue().
                     elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmFullDmgSurvProbB.GetValue().
                      elementAt(1)).floatValue(), 0, 0.001);

      assertEquals( ( (Float) oDist.mp_fStmPropFullTipUp.GetValue().elementAt(0)).
                   floatValue(), 0.5, 0.001);
      assertEquals( ( (Float) oDist.mp_fStmPropFullTipUp.GetValue().elementAt(1)).
                   floatValue(), 0.6, 0.001);

      assertEquals(4, oDist.m_iNumYearsToHeal.GetValue());
      assertEquals(3, oDist.m_iNumSnagYears.GetValue());

      new File(sFileName).delete();
    }
    catch (ModelException oErr) {
      fail("Storm validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests ValidateData().  Four valid files and 10 invalid files are
   * put through.
   * @throws ModelException if the validation happens incorrectly.
   */
  public void testValidateData() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 2
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 3
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 4
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 5
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

      //Windstorm valid file
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Disturbance validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 4
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 4.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 5
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 5.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 6
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile6();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 6.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 7
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile7();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 7.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 8
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile8();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 8.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 9
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLErrorFile9();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetDisturbanceBehaviors().ValidateData(oManager.
          GetTreePopulation());
      fail("Didn't catch error in error file 9.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 1 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt1Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 5 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt5Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 5.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }


    //Exception processing - windstorms - return interval 10 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt10Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 10.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 20 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt20Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 20.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 40 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt40Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 40.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 80 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt80Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 80.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 160 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt160Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 160.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 320 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt320Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 320.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 640 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt640Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 640.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 1280 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt1280Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 1280.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - return interval 2560 not proportion
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormReturnInt2560Severity.SetValue((float)1.1);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch bad value in windstorm return interval 2560.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - timestep to start storms is negative
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_iWindstormTimestepToStartStorms.SetValue(-12);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch negative timestep to start storms.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - min DBH negative
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.mp_fWindstormMinDBH.GetValue().clear();
      oDist.mp_fWindstormMinDBH.GetValue().add(new Float(-1.2));
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch negative min DBH.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - mortality behaviors not applied
    //to all trees
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.GetBehaviorByKey("Windstorm").AddSpeciesTypeCombo(
         new SpeciesTypeCombo(0, TreePopulation.SAPLING,
                              oManager.GetTreePopulation()));
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch mortality not applied to all trees.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - windstorms - windstorm cyclicity is 0
    try {
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidWindstormFile();
      oManager.InputXMLParameterFile(sFileName);
      DisturbanceBehaviors oDist = oManager.GetDisturbanceBehaviors();
      oDist.m_fWindstormSSTPeriod.SetValue(0);
      oDist.ValidateData(oManager.GetTreePopulation());
      fail("Didn't SST periodicity = 0.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }




    System.out.println("Storm ValidateData test was successful.");

  }

  /**
   * Writes a valid file for storms with deterministic uniform damage.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  protected String WriteXMLValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    WriteStormParameters(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid file for storms with stochastic uniform damage.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  protected String WriteXMLValidFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    WriteStormParameters(oOut);
    oOut.write("<st_stochasticity>1</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("<st_probFunction>0</st_probFunction>");
    oOut.write("<st_standardDeviation>0.3</st_standardDeviation>");
    oOut.write("<st_stmSSTPeriod>50</st_stmSSTPeriod>");
    oOut.write("<st_stmSineD>1</st_stmSineD>");
    oOut.write("<st_stmSineF>2</st_stmSineF>");
    oOut.write("<st_stmSineG>3</st_stmSineG>");
    oOut.write("<st_stmTrendSlopeM>4</st_stmTrendSlopeM>");
    oOut.write("<st_stmTrendInterceptI>-4</st_stmTrendInterceptI>");
    oOut.write("</storm>");
    oOut.write("</paramFile>"); ;
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid file for storms with stochastic uniform damage and a
   * negative binomial function.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  protected String WriteXMLValidFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    WriteStormParameters(oOut);
    oOut.write("<st_stochasticity>1</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("<st_probFunction>1</st_probFunction>");
    oOut.write("</storm>");
    oOut.write("</paramFile>"); ;
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file for a uniform mapped damage pattern.  There is a
   * grid map for "Storm Susceptibility".
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLValidFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    //Generate the storm susceptibility map
    oOut.write("<grid gridName=\"Storm Susceptibility\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"index\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>5</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>3</ma_lengthYCells>");

    //Auto-generate grid values
    float fCount = 0;
    int iNumX = 20, iNumY = 44, iX, iY;
    for (iX = 0; iX < iNumX; iX++) {
      for (iY = 0; iY < iNumY; iY++) {
        String sVal = "<ma_v x=\"" + iX + "\" y=\"" + iY + "\"><fl c=\"0\">" +
            fCount + "</fl></ma_v>";
        oOut.write(sVal);
        fCount += 0.001;
      }
    }
    oOut.write("</grid>");
    WriteBehaviorList(oOut);
    WriteStormParameters(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>0</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file for a stochastic mapped damage pattern.  There is a
   * grid map for "Storm Susceptibility".
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLValidFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    //Generate the storm susceptibility map
    oOut.write("<grid gridName=\"Storm Susceptibility\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"index\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>5</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>3</ma_lengthYCells>");

    //Auto-generate grid values
    float fCount = 0;
    int iNumX = 20, iNumY = 44, iX, iY;
    for (iX = 0; iX < iNumX; iX++) {
      for (iY = 0; iY < iNumY; iY++) {
        String sVal = "<ma_v x=\"" + iX + "\" y=\"" + iY + "\"><fl c=\"0\">" +
            fCount + "</fl></ma_v>";
        oOut.write(sVal);
        fCount += 0.001;
      }
    }
    oOut.write("</grid>");
    WriteBehaviorList(oOut);
    WriteStormParameters(oOut);
    oOut.write("<st_stochasticity>1</st_stochasticity>");
    oOut.write("<st_susceptibility>0</st_susceptibility>");
    oOut.write("<st_probFunction>1</st_probFunction>");
    oOut.write("<st_standardDeviation>0.3</st_standardDeviation>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid file for windstorms.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 19, 2006: Created (LEM)
   */
  protected String WriteXMLValidWindstormFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Windstorm</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>adultstochasticmort</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_nonSizeDepMort>");
    oOut.write("<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>");
    oOut.write("<mo_nsdmVal species=\"Species_2\">0.0</mo_nsdmVal>");
    oOut.write("</mo_nonSizeDepMort>");
    oOut.write("</mortality>");
    oOut.write("<windstorm>");
    oOut.write("<ws_severityReturnInterval1>0.10</ws_severityReturnInterval1>");
    oOut.write("<ws_severityReturnInterval5>0.11</ws_severityReturnInterval5>");
    oOut.write("<ws_severityReturnInterval10>0.12</ws_severityReturnInterval10>");
    oOut.write("<ws_severityReturnInterval20>0.13</ws_severityReturnInterval20>");
    oOut.write("<ws_severityReturnInterval40>0.14</ws_severityReturnInterval40>");
    oOut.write("<ws_severityReturnInterval80>0.15</ws_severityReturnInterval80>");
    oOut.write("<ws_severityReturnInterval160>0.16</ws_severityReturnInterval160>");
    oOut.write("<ws_severityReturnInterval320>0.17</ws_severityReturnInterval320>");
    oOut.write("<ws_severityReturnInterval640>0.18</ws_severityReturnInterval640>");
    oOut.write("<ws_severityReturnInterval1280>0.19</ws_severityReturnInterval1280>");
    oOut.write("<ws_severityReturnInterval2560>0.2</ws_severityReturnInterval2560>");
    oOut.write("<ws_minDBH>");
    oOut.write("<ws_mdVal species=\"Species_1\">5</ws_mdVal>");
    oOut.write("<ws_mdVal species=\"Species_2\">6</ws_mdVal>");
    oOut.write("</ws_minDBH>");
    oOut.write("<ws_stmInterceptA>");
    oOut.write("<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>");
    oOut.write("<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>");
    oOut.write("</ws_stmInterceptA>");
    oOut.write("<ws_stmIntensCoeffC>");
    oOut.write("<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>");
    oOut.write("<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>");
    oOut.write("</ws_stmIntensCoeffC>");
    oOut.write("<ws_stmDBHExpB>");
    oOut.write("<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>");
    oOut.write("<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>");
    oOut.write("</ws_stmDBHExpB>");
    oOut.write("<ws_stmTSToStartStorms>1</ws_stmTSToStartStorms>");
    oOut.write("<ws_stmSSTPeriod>50</ws_stmSSTPeriod>");
    oOut.write("<ws_stmSineD>0</ws_stmSineD>");
    oOut.write("<ws_stmSineF>1</ws_stmSineF>");
    oOut.write("<ws_stmSineG>2</ws_stmSineG>");
    oOut.write("<ws_stmTrendSlopeM>3</ws_stmTrendSlopeM>");
    oOut.write("<ws_stmTrendInterceptI>-4</ws_stmTrendInterceptI>");
    oOut.write("</windstorm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a 6.05 file.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 9, 2006: Created (LEM)
   */
  protected String Write6_05XMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm damage applier</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmPropDieMedDmg>");
    oOut.write("<st_spdmdVal species=\"Species_1\">0.1</st_spdmdVal>");
    oOut.write("<st_spdmdVal species=\"Species_2\">0.2</st_spdmdVal>");
    oOut.write("</st_stmPropDieMedDmg>");
    oOut.write("<st_stmPropDieFullDmg>");
    oOut.write("<st_spdfdVal species=\"Species_1\">0.3</st_spdfdVal>");
    oOut.write("<st_spdfdVal species=\"Species_2\">0</st_spdfdVal>");
    oOut.write("</st_stmPropDieFullDmg>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }


  /**
   * Writes a parameter file where there are grid maps for both "Storm Damage"
   * and "Storm Susceptibility" and the grid cell resolutions don't match.
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    //Generate the storm susceptibility map
    oOut.write("<grid gridName=\"Storm Susceptibility\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"index\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>5</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>3</ma_lengthYCells>");

    //Auto-generate grid values
    int iNumX = 20, iNumY = 44, iX, iY;
    for (iX = 0; iX < iNumX; iX++) {
      for (iY = 0; iY < iNumY; iY++) {
        String sVal = "<ma_v x=\"" + iX + "\" y=\"" + iY + "\"><fl c=\"0\">" +
            0.5 + "</fl></ma_v>";
        oOut.write(sVal);
      }
    }
    oOut.write("</grid>");
    oOut.write("<grid gridName=\"Storm Damage\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"dmg_index\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>10</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>10</ma_lengthYCells>");
    oOut.write("</grid>");
    WriteBehaviorList(oOut);
    WriteStormParameters(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>0</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>"); ;
    oOut.close();
    return sFileName;

  }

  /**
   * Writes a parameter file where the damage pattern is set to "mapped" but
   * there is no "Storm Susceptibility" grid map in the parameter file.
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    WriteStormParameters(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>0</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file where not all storm return interval values are
   * positive.
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>-1.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a parameter file where the storm damage applier behavior is enabled
   * but not the storm behavior.
   * @return Filename written.
   * @throws java.io.IOException if there is a problem writing the file.
   */
  protected String WriteXMLErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm damage applier</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<storm>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the number of years to heal is negative.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 29, 2004: Created (LEM)
   */
  protected String WriteXMLErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmPropDieMedDmg>");
    oOut.write("<st_spdmdVal species=\"Species_1\">0.1</st_spdmdVal>");
    oOut.write("<st_spdmdVal species=\"Species_2\">0.2</st_spdmdVal>");
    oOut.write("</st_stmPropDieMedDmg>");
    oOut.write("<st_stmPropDieFullDmg>");
    oOut.write("<st_spdfdVal species=\"Species_1\">0.3</st_spdfdVal>");
    oOut.write("<st_spdfdVal species=\"Species_2\">0.4</st_spdfdVal>");
    oOut.write("</st_stmPropDieFullDmg>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>-4</st_numYearsToHeal>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the "Storm Killer" behavior is enabled but not
   * the "Storm Damage Applier".
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 4, 2005: Created (LEM)
   */
  protected String WriteXMLErrorFile6() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm killer</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmMedDmgSurvProbA>");
    oOut.write("<st_smdspaVal species=\"Species_1\">0.1</st_smdspaVal>");
    oOut.write("<st_smdspaVal species=\"Species_2\">0.2</st_smdspaVal>");
    oOut.write("</st_stmMedDmgSurvProbA>");
    oOut.write("<st_stmMedDmgSurvProbB>");
    oOut.write("<st_smdspbVal species=\"Species_1\">0.3</st_smdspbVal>");
    oOut.write("<st_smdspbVal species=\"Species_2\">0.4</st_smdspbVal>");
    oOut.write("</st_stmMedDmgSurvProbB>");
    oOut.write("<st_stmFullDmgSurvProbA>");
    oOut.write("<st_sfdspaVal species=\"Species_1\">-0.1</st_sfdspaVal>");
    oOut.write("<st_sfdspaVal species=\"Species_2\">-0.2</st_sfdspaVal>");
    oOut.write("</st_stmFullDmgSurvProbA>");
    oOut.write("<st_stmFullDmgSurvProbB>");
    oOut.write("<st_sfdspbVal species=\"Species_1\">-0.3</st_sfdspbVal>");
    oOut.write("<st_sfdspbVal species=\"Species_2\">-0.4</st_sfdspbVal>");
    oOut.write("</st_stmFullDmgSurvProbB>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the species/type combos assigned to "Storm Damage
   * Applier" and "Storm Killer" don't match.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 4, 2005: Created (LEM)
   */
  protected String WriteXMLErrorFile7() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm damage applier</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm killer</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    WriteStormParameters(oOut);
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the proportion of trees with total damage that tip up
   * is not a proportion.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>May 2, 2005: Created (LEM)
   */
  protected String WriteXMLErrorFile8() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmMedDmgSurvProbA>");
    oOut.write("<st_smdspaVal species=\"Species_1\">0.1</st_smdspaVal>");
    oOut.write("<st_smdspaVal species=\"Species_2\">0.2</st_smdspaVal>");
    oOut.write("</st_stmMedDmgSurvProbA>");
    oOut.write("<st_stmMedDmgSurvProbB>");
    oOut.write("<st_smdspbVal species=\"Species_1\">0.3</st_smdspbVal>");
    oOut.write("<st_smdspbVal species=\"Species_2\">0.4</st_smdspbVal>");
    oOut.write("</st_stmMedDmgSurvProbB>");
    oOut.write("<st_stmFullDmgSurvProbA>");
    oOut.write("<st_sfdspaVal species=\"Species_1\">-0.1</st_sfdspaVal>");
    oOut.write("<st_sfdspaVal species=\"Species_2\">-0.2</st_sfdspaVal>");
    oOut.write("</st_stmFullDmgSurvProbA>");
    oOut.write("<st_stmFullDmgSurvProbB>");
    oOut.write("<st_sfdspbVal species=\"Species_1\">-0.3</st_sfdspbVal>");
    oOut.write("<st_sfdspbVal species=\"Species_2\">-0.4</st_sfdspbVal>");
    oOut.write("</st_stmFullDmgSurvProbB>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">1.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the snag lifetime is negative.
   * @return The file name.
   * @throws java.io.IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>May 2, 2005: Created (LEM)
   */
  protected String WriteXMLErrorFile9() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteFileStart(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmMedDmgSurvProbA>");
    oOut.write("<st_smdspaVal species=\"Species_1\">0.1</st_smdspaVal>");
    oOut.write("<st_smdspaVal species=\"Species_2\">0.2</st_smdspaVal>");
    oOut.write("</st_stmMedDmgSurvProbA>");
    oOut.write("<st_stmMedDmgSurvProbB>");
    oOut.write("<st_smdspbVal species=\"Species_1\">0.3</st_smdspbVal>");
    oOut.write("<st_smdspbVal species=\"Species_2\">0.4</st_smdspbVal>");
    oOut.write("</st_stmMedDmgSurvProbB>");
    oOut.write("<st_stmFullDmgSurvProbA>");
    oOut.write("<st_sfdspaVal species=\"Species_1\">-0.1</st_sfdspaVal>");
    oOut.write("<st_sfdspaVal species=\"Species_2\">-0.2</st_sfdspaVal>");
    oOut.write("</st_stmFullDmgSurvProbA>");
    oOut.write("<st_stmFullDmgSurvProbB>");
    oOut.write("<st_sfdspbVal species=\"Species_1\">-0.3</st_sfdspbVal>");
    oOut.write("<st_sfdspbVal species=\"Species_2\">-0.4</st_sfdspbVal>");
    oOut.write("</st_stmFullDmgSurvProbB>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>-3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
    oOut.write("<st_stochasticity>0</st_stochasticity>");
    oOut.write("<st_susceptibility>1</st_susceptibility>");
    oOut.write("</storm>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes beginning portion of test parameter files.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteFileStart(FileWriter oOut) throws java.io.IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

  /**
   * Writes storm behavior list.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteBehaviorList(FileWriter oOut) throws java.io.IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm damage applier</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Snag\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>storm killer</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  /**
   * Writes storm parameters.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteStormParameters(FileWriter oOut) throws java.io.IOException {
    oOut.write("<storm>");
    oOut.write("<st_s1ReturnInterval>0.1</st_s1ReturnInterval>");
    oOut.write("<st_s2ReturnInterval>0.2</st_s2ReturnInterval>");
    oOut.write("<st_s3ReturnInterval>0.3</st_s3ReturnInterval>");
    oOut.write("<st_s4ReturnInterval>0.4</st_s4ReturnInterval>");
    oOut.write("<st_s5ReturnInterval>0.5</st_s5ReturnInterval>");
    oOut.write("<st_s6ReturnInterval>0.6</st_s6ReturnInterval>");
    oOut.write("<st_s7ReturnInterval>0.7</st_s7ReturnInterval>");
    oOut.write("<st_s8ReturnInterval>0.8</st_s8ReturnInterval>");
    oOut.write("<st_s9ReturnInterval>0.9</st_s9ReturnInterval>");
    oOut.write("<st_s10ReturnInterval>1</st_s10ReturnInterval>");
    oOut.write("<st_minDBH>");
    oOut.write("<st_mdVal species=\"Species_1\">5</st_mdVal>");
    oOut.write("<st_mdVal species=\"Species_2\">6</st_mdVal>");
    oOut.write("</st_minDBH>");
    oOut.write("<st_stmDmgInterceptMed>");
    oOut.write("<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>");
    oOut.write("<st_sdimVal species=\"Species_2\">2.772772</st_sdimVal>");
    oOut.write("</st_stmDmgInterceptMed>");
    oOut.write("<st_stmDmgInterceptFull>");
    oOut.write("<st_sdifVal species=\"Species_1\">2.6334169</st_sdifVal>");
    oOut.write("<st_sdifVal species=\"Species_2\">4.9189817</st_sdifVal>");
    oOut.write("</st_stmDmgInterceptFull>");
    oOut.write("<st_stmIntensityCoeff>");
    oOut.write("<st_sicVal species=\"Species_1\">-0.2545352</st_sicVal>");
    oOut.write("<st_sicVal species=\"Species_2\">-1.936994</st_sicVal>");
    oOut.write("</st_stmIntensityCoeff>");
    oOut.write("<st_stmDBHCoeff>");
    oOut.write("<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>");
    oOut.write("<st_sdcVal species=\"Species_2\">0.2411269</st_sdcVal>");
    oOut.write("</st_stmDBHCoeff>");
    oOut.write("<st_stmMedDmgSurvProbA>");
    oOut.write("<st_smdspaVal species=\"Species_1\">0.1</st_smdspaVal>");
    oOut.write("<st_smdspaVal species=\"Species_2\">0.2</st_smdspaVal>");
    oOut.write("</st_stmMedDmgSurvProbA>");
    oOut.write("<st_stmMedDmgSurvProbB>");
    oOut.write("<st_smdspbVal species=\"Species_1\">0.3</st_smdspbVal>");
    oOut.write("<st_smdspbVal species=\"Species_2\">0.4</st_smdspbVal>");
    oOut.write("</st_stmMedDmgSurvProbB>");
    oOut.write("<st_stmFullDmgSurvProbA>");
    oOut.write("<st_sfdspaVal species=\"Species_1\">-0.1</st_sfdspaVal>");
    oOut.write("<st_sfdspaVal species=\"Species_2\">-0.2</st_sfdspaVal>");
    oOut.write("</st_stmFullDmgSurvProbA>");
    oOut.write("<st_stmFullDmgSurvProbB>");
    oOut.write("<st_sfdspbVal species=\"Species_1\">-0.3</st_sfdspbVal>");
    oOut.write("<st_sfdspbVal species=\"Species_2\">-0.4</st_sfdspbVal>");
    oOut.write("</st_stmFullDmgSurvProbB>");
    oOut.write("<st_stmPropTipUpFullDmg>");
    oOut.write("<st_sptufdVal species=\"Species_1\">0.5</st_sptufdVal>");
    oOut.write("<st_sptufdVal species=\"Species_2\">0.6</st_sptufdVal>");
    oOut.write("</st_stmPropTipUpFullDmg>");
    oOut.write("<st_numYearsStormSnags>3</st_numYearsStormSnags>");
    oOut.write("<st_numYearsToHeal>4</st_numYearsToHeal>");
  }

  /**
   * Writes a file with no disturbance behaviors.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteNeutralFile() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">45</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">0.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

}
